import subprocess
from unittest import mock

import pytest


@pytest.mark.parametrize("is_color_enabled", [True, False])
def test_logcat(mock_tools, adb, is_color_enabled, monkeypatch):
    """Invoking `logcat()` calls `Popen()` with the appropriate parameters."""
    # Mock whether color is enabled for the console
    monkeypatch.setattr(
        type(mock_tools.input),
        "is_color_enabled",
        mock.PropertyMock(return_value=is_color_enabled),
    )

    # Mock the result of calling Popen so we can compare against this return value
    popen = mock.MagicMock()
    mock_tools.subprocess.Popen.return_value = popen

    # Invoke logcat
    result = adb.logcat("1234")

    # Validate call parameters.
    mock_tools.subprocess.Popen.assert_called_once_with(
        [
            mock_tools.android_sdk.adb_path,
            "-s",
            "exampleDevice",
            "logcat",
            "--format=tag",
            "--pid",
            "1234",
            "EGL_emulation:S",
        ]
        + (["--format=color"] if is_color_enabled else []),
        env=mock_tools.android_sdk.env,
        encoding="UTF-8",
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        bufsize=1,
    )

    # The Popen object is returned
    assert result == popen



from unittest.mock import MagicMock

from ...utils import NoMatchString, PartialMatchString


def test_override_is_used(convert_command):
    """The override value is used even if PEP621 data is present."""
    (convert_command.base_path / "pyproject.toml").write_text(
        '[project]\nname="test-name"', encoding="utf-8"
    )
    assert convert_command.input_app_name("OVERRIDE") == "OVERRIDE"


def test_no_pep621_data(convert_command, monkeypatch):
    """The app directory is used if there is no PEP621-name."""
    mock_input_text = MagicMock()
    monkeypatch.setattr(convert_command, "input_text", mock_input_text)
    convert_command.base_path /= "test-app-name"
    convert_command.input_app_name(None)

    mock_input_text.assert_called_once_with(
        intro=PartialMatchString(
            "Based on your PEP508 formatted directory name, we suggest an app name of 'test-app-name'"
        ),
        variable="app name",
        default="test-app-name",
        validator=convert_command.validate_app_name,
        override_value=None,
    )


def test_valid_pep621_app_name(convert_command):
    """The PEP621-name is used if present and valid."""
    (convert_command.base_path / "pyproject.toml").write_text(
        '[project]\nname="test-name"', encoding="utf-8"
    )
    assert convert_command.input_app_name(None) == "test-name"


def test_pep621_name_is_canonicalized(convert_command):
    (convert_command.base_path / "pyproject.toml").write_text(
        '[project]\nname="test.name"', encoding="utf-8"
    )
    assert convert_command.input_app_name(None) == "test-name"


def test_invalid_hint_app_name(convert_command, monkeypatch):
    """A placeholder is used if there's no PEP621 name and the app directory is an
    invalid name."""
    mock_input_text = MagicMock()
    monkeypatch.setattr(convert_command, "input_text", mock_input_text)
    convert_command.base_path /= "!app_name"
    convert_command.input_app_name(None)

    mock_input_text.assert_called_once_with(
        intro=NoMatchString(
            "Based on your PEP508 formatted directory name, we suggest an app name of 'test-app-name'"
        ),
        variable="app name",
        default="hello-world",
        validator=convert_command.validate_app_name,
        override_value=None,
    )


def test_hint_is_canonicalized(convert_command, monkeypatch):
    """The app directory name is canonicalized when used as a hint."""
    mock_input_text = MagicMock()
    monkeypatch.setattr(convert_command, "input_text", mock_input_text)
    convert_command.base_path /= "test-app_name"
    convert_command.input_app_name(None)

    mock_input_text.assert_called_once_with(
        intro=PartialMatchString(
            "Based on your PEP508 formatted directory name, we suggest an app name of 'test-app-name'"
        ),
        variable="app name",
        default="test-app-name",
        validator=convert_command.validate_app_name,
        override_value=None,
    )



from automation.bootstraps import BRIEFCASE_EXIT_SUCCESS_SIGNAL, EXIT_SUCCESS_NOTIFY
from briefcase.bootstraps import ConsoleBootstrap


class ConsoleAutomationBootstrap(ConsoleBootstrap):
    def app_source(self):
        return f"""\
import time


def main():
    time.sleep(2)
    print("{EXIT_SUCCESS_NOTIFY}")
    print("{BRIEFCASE_EXIT_SUCCESS_SIGNAL}")
"""



from unittest.mock import Mock

from briefcase.console import Console


def test_console_is_controlled():
    """Console control is released and then restored."""
    console = Console()

    console.is_console_controlled = True

    with console.release_console_control():
        assert console.is_console_controlled is False

    assert console.is_console_controlled is True


def test_console_is_not_controlled():
    """Effective no-op when console is not currently controlled."""
    console = Console()

    assert console.is_console_controlled is False

    with console.release_console_control():
        assert console.is_console_controlled is False

    assert console.is_console_controlled is False


def test_wait_bar_is_active(console):
    """An active Wait Bar is stopped."""
    with console.wait_bar("Testing..."):
        # Wrap the Wait Bar stop and start methods
        console._wait_bar.stop = Mock(wraps=console._wait_bar.stop)
        console._wait_bar.start = Mock(wraps=console._wait_bar.start)

        # Wait Bar is active
        assert console._wait_bar.live.is_started is True
        assert console.is_console_controlled is True

        with console.release_console_control():
            # Wait Bar is not active
            console._wait_bar.stop.assert_called_once()
            assert console._wait_bar.live.is_started is False
            assert console.is_console_controlled is False

        # Wait bar is restored and active again
        console._wait_bar.stop.assert_called_once()
        assert console._wait_bar.live.is_started is True
        assert console.is_console_controlled is True


def test_wait_bar_is_not_active():
    """An inactive Wait Bar is not effected."""
    console = Console()

    # Instantiate the Wait Bar
    with console.wait_bar("Testing..."):
        pass

    # Mock the Wait Bar stop and start methods
    console._wait_bar.stop = Mock(wraps=console._wait_bar.stop)
    console._wait_bar.start = Mock(wraps=console._wait_bar.start)

    assert console._wait_bar.live.is_started is False
    assert console.is_console_controlled is False

    with console.release_console_control():
        # A stop request was not made for the Wait Bar
        console._wait_bar.stop.assert_not_called()
        assert console._wait_bar.live.is_started is False
        assert console.is_console_controlled is False

    # A start request was not made for the Wait Bar
    console._wait_bar.start.assert_not_called()
    assert console._wait_bar.live.is_started is False
    assert console.is_console_controlled is False



import subprocess
from pathlib import Path
from unittest import mock

import pytest

import briefcase.platforms.windows.visualstudio
from briefcase.console import Console, Log, LogLevel
from briefcase.exceptions import BriefcaseCommandError
from briefcase.integrations.subprocess import Subprocess
from briefcase.integrations.visualstudio import VisualStudio
from briefcase.platforms.windows.visualstudio import WindowsVisualStudioBuildCommand


@pytest.fixture
def build_command(tmp_path):
    command = WindowsVisualStudioBuildCommand(
        logger=Log(),
        console=Console(),
        base_path=tmp_path / "base_path",
        data_path=tmp_path / "briefcase",
    )
    command.tools.subprocess = mock.MagicMock(spec_set=Subprocess)
    command.tools.visualstudio = VisualStudio(
        tools=command.tools,
        msbuild_path=tmp_path / "Visual Studio/MSBuild.exe",
    )
    return command


def test_verify(build_command, monkeypatch):
    """Verifying on Windows creates a VisualStudio wrapper."""
    build_command.tools.host_os = "Windows"

    mock_visualstudio_verify = mock.MagicMock(wraps=VisualStudio.verify)
    monkeypatch.setattr(
        briefcase.platforms.windows.visualstudio.VisualStudio,
        "verify",
        mock_visualstudio_verify,
    )

    build_command.verify_tools()

    # VisualStudio tool was verified
    mock_visualstudio_verify.assert_called_once_with(tools=build_command.tools)
    assert isinstance(build_command.tools.visualstudio, VisualStudio)


@pytest.mark.parametrize("tool_debug_mode", (True, False))
def test_build_app(build_command, first_app_config, tool_debug_mode, tmp_path):
    """The solution will be compiled when the project is built."""
    # Enable verbose tool logging
    if tool_debug_mode:
        build_command.tools.logger.verbosity = LogLevel.DEEP_DEBUG

    build_command.build_app(first_app_config)

    build_command.tools.subprocess.run.assert_has_calls(
        [
            # Collect manifest
            mock.call(
                [
                    Path(tmp_path) / "Visual Studio/MSBuild.exe",
                    "First App.sln",
                    "-target:restore",
                    "-property:RestorePackagesConfig=true",
                    "-target:First App",
                    "-property:Configuration=Release",
                    "-verbosity:detailed" if tool_debug_mode else "-verbosity:normal",
                ],
                check=True,
                cwd=tmp_path
                / "base_path"
                / "build"
                / "first-app"
                / "windows"
                / "visualstudio",
            ),
        ]
    )


def test_build_app_failure(build_command, first_app_config, tmp_path):
    """If the stub binary cannot be updated, an error is raised."""

    build_command.tools.subprocess.run.side_effect = subprocess.CalledProcessError(
        returncode=1,
        cmd="MSBuild.exe",
    )

    with pytest.raises(
        BriefcaseCommandError,
        match=r"Unable to build solution for first-app.",
    ):
        build_command.build_app(first_app_config)



from automation.bootstraps import BRIEFCASE_EXIT_SUCCESS_SIGNAL, EXIT_SUCCESS_NOTIFY
from briefcase.bootstraps import TogaGuiBootstrap


class TogaAutomationBootstrap(TogaGuiBootstrap):
    def app_source(self):
        return f'''\
import asyncio

import toga
from toga.style import Pack
from toga.style.pack import COLUMN, ROW


class {{{{ cookiecutter.class_name }}}}(toga.App):
    def startup(self):
        """Construct and show the Toga application.

        Usually, you would add your application to a main content box.
        We then create a main window (with a name matching the app), and
        show the main window.
        """
        main_box = toga.Box()

        self.main_window = toga.MainWindow(title=self.formal_name)
        self.main_window.content = main_box
        self.main_window.show()

        self.add_background_task(self.exit_soon)

    async def exit_soon(self, app: toga.App, **kwargs):
        """Background task that closes the app after a few seconds."""
        await asyncio.sleep(2)
        print("{EXIT_SUCCESS_NOTIFY}")
        print("{BRIEFCASE_EXIT_SUCCESS_SIGNAL}")
        self.exit()


def main():
    return {{{{ cookiecutter.class_name }}}}()
'''



from __future__ import annotations

from pathlib import Path

from briefcase.commands import (
    BuildCommand,
    CreateCommand,
    OpenCommand,
    PackageCommand,
    PublishCommand,
    RunCommand,
    UpdateCommand,
)
from briefcase.config import AppConfig
from briefcase.platforms.macOS import (
    SigningIdentity,
    macOSCreateMixin,
    macOSMixin,
    macOSPackageMixin,
    macOSRunMixin,
    macOSSigningMixin,
)
from briefcase.platforms.macOS.utils import AppPackagesMergeMixin


class macOSAppMixin(macOSMixin):
    output_format = "app"

    def project_path(self, app):
        return self.binary_path(app) / "Contents"

    def binary_path(self, app):
        return self.bundle_path(app) / f"{app.formal_name}.app"

    def binary_executable_path(self, app) -> Path:
        # The actual binary in a macOS app is a known path
        # inside the "binary" app bundle that is executed.
        return self.binary_path(app) / "Contents/MacOS" / app.formal_name


class macOSAppCreateCommand(macOSAppMixin, macOSCreateMixin, CreateCommand):
    description = "Create and populate a macOS app."

    def support_path(self, app: AppConfig, runtime=False) -> Path:
        if runtime:
            return super().support_path(app)
        else:
            return self.bundle_path(app) / "support"

    def install_app_support_package(self, app: AppConfig):
        """Install the application support package.

        :param app: The config object for the app
        """
        super().install_app_support_package(app)

        # Copy the stdlib into its final location
        with self.input.wait_bar("Copying standard library into app bundle..."):
            runtime_support_path = self.support_path(app, runtime=True)
            if runtime_support_path.is_dir():
                self.tools.shutil.rmtree(runtime_support_path)
            runtime_support_path.mkdir()

            self.tools.shutil.copytree(
                self.support_path(app) / "python-stdlib",
                runtime_support_path / "python-stdlib",
            )

    def install_app_resources(self, app: AppConfig):
        super().install_app_resources(app)

        # macOS will cache application icons. Touching the .app folder flushes the icon
        # cache for the app, ensuring the current icon is loaded.
        self.binary_path(app).touch(exist_ok=True)


class macOSAppUpdateCommand(macOSAppCreateCommand, UpdateCommand):
    description = "Update an existing macOS app."


class macOSAppOpenCommand(macOSAppMixin, OpenCommand):
    description = "Open the app bundle folder for an existing macOS app."


class macOSAppBuildCommand(
    macOSAppMixin,
    macOSSigningMixin,
    AppPackagesMergeMixin,
    BuildCommand,
):
    description = "Build a macOS app."

    def build_app(self, app: AppConfig, **kwargs):
        """Build the macOS app.

        :param app: The application to build
        """
        self.logger.info("Building App...", prefix=app.app_name)

        # Move the unbuilt binary in to the final executable location
        unbuilt_path = self.unbuilt_executable_path(app)
        if unbuilt_path.exists():
            with self.input.wait_bar("Renaming stub binary..."):
                unbuilt_path.rename(self.binary_executable_path(app))

        if not getattr(app, "universal_build", True):
            with self.input.wait_bar("Ensuring stub binary is thin..."):
                # The stub binary is universal by default. If we're building a non-universal app,
                # we can strip the binary to remove the unused slice. This occurs before the
                self.ensure_thin_binary(
                    self.binary_executable_path(app),
                    arch=self.tools.host_arch,
                )

        # macOS apps don't have anything to compile, but they do need to be
        # signed to be able to execute on Apple Silicon hardware - even if it's only an
        # ad-hoc signing identity. Apply an ad-hoc signing identity to the
        # app bundle.
        self.logger.info("Ad-hoc signing app...", prefix=app.app_name)
        self.sign_app(app=app, identity=SigningIdentity())


class macOSAppRunCommand(macOSRunMixin, macOSAppMixin, RunCommand):
    description = "Run a macOS app."


class macOSAppPackageCommand(macOSPackageMixin, macOSAppMixin, PackageCommand):
    description = "Package a macOS app for distribution."


class macOSAppPublishCommand(macOSAppMixin, PublishCommand):
    description = "Publish a macOS app."


# Declare the briefcase command bindings
create = macOSAppCreateCommand
update = macOSAppUpdateCommand
open = macOSAppOpenCommand
build = macOSAppBuildCommand
run = macOSAppRunCommand
package = macOSAppPackageCommand
publish = macOSAppPublishCommand



import pytest


@pytest.mark.parametrize(
    "app_name, bundle, candidate",
    [
        ("some-app", "com.example", "https://example.com/some-app"),
        ("some_app", "com.example.more", "https://more.example.com/some_app"),
        ("myapp", "com.example", "https://example.com/myapp"),
    ],
)
def test_make_project_url(new_command, app_name, bundle, candidate):
    """An app name and bundle can be converted into a valid URL."""
    url = new_command.make_project_url(bundle, app_name)
    assert url == candidate
    # Double check - the app name passes the validity check.
    assert new_command.validate_url(url)



from briefcase.config import merge_config


def test_merge_no_options_no_data():
    """If there are no initial options or new additional options, nothing changes."""
    config = {"other": 1234}

    merge_config(config, {})

    assert config == {"other": 1234}


def test_merge_no_data():
    """If there are no new options, nothing changes."""
    config = {
        "requires": ["first", "second"],
        "permission": {"up": True, "down": False},
        "other": 1234,
    }

    merge_config(config, {})

    assert config == {
        "requires": ["first", "second"],
        "permission": {"up": True, "down": False},
        "other": 1234,
    }


def test_merge_no_option():
    """If there are no existing options, the new option become the entire value."""
    config = {"other": 1234}

    merge_config(
        config,
        {
            "requires": ["third", "fourth"],
            "permission": {"left": True, "right": False},
        },
    )

    assert config == {
        "requires": ["third", "fourth"],
        "permission": {"left": True, "right": False},
        "other": 1234,
    }


def test_merge():
    """If there are existing options and new options, merge."""
    config = {
        "requires": ["first", "second"],
        "permission": {"up": True, "down": False},
        "other": 1234,
    }

    merge_config(
        config,
        {
            "requires": ["third", "fourth"],
            "permission": {"left": True, "right": False},
            "other": 5678,
        },
    )

    assert config == {
        "requires": ["first", "second", "third", "fourth"],
        "permission": {"up": True, "down": False, "left": True, "right": False},
        "other": 5678,
    }


def test_merge_collision():
    """If there are repeated options, lists are appended, dictionaries are updated and
    new options, merge."""
    config = {
        "requires": ["first", "second"],
        "permission": {"up": True, "down": False},
        "other": 1234,
    }

    merge_config(
        config,
        {
            "requires": ["second", "fourth"],
            "permission": {"down": True, "right": False},
            "other": 5678,
        },
    )

    assert config == {
        "requires": ["first", "second", "second", "fourth"],
        "permission": {"up": True, "down": True, "right": False},
        "other": 5678,
    }


def test_convert_base_definition():
    """The merge operation succeeds when called on itself."""
    config = {
        "requires": ["first", "second"],
        "permission": {"up": True, "down": False},
        "other": 1234,
    }

    merge_config(config, config)

    assert config == {
        "requires": ["first", "second"],
        "permission": {"up": True, "down": False},
        "other": 1234,
    }


def test_merged_keys():
    """There are multiple mergeable keys."""
    config = {
        "requires": ["first", "second"],
        "sources": ["a", "b"],
        "permission": {"up": True, "down": False},
        "non-merge": ["1", "2"],
        "other": 1234,
    }

    merge_config(
        config,
        {
            "requires": ["third", "fourth"],
            "permission": {"left": True, "right": False},
            "sources": ["c", "d"],
            "non-merge": ["3", "4"],
        },
    )

    assert config == {
        "requires": ["first", "second", "third", "fourth"],
        "sources": ["a", "b", "c", "d"],
        "permission": {"up": True, "down": False, "left": True, "right": False},
        "non-merge": ["3", "4"],
        "other": 1234,
    }



import os
from pathlib import Path

import pytest

from briefcase.exceptions import BriefcaseCommandError

from ....utils import assert_url_resolvable
from ..conftest import SDK_MGR_DL_VER, SDK_MGR_VER


@pytest.mark.parametrize(
    "host_os, host_arch, name",
    [
        ("Darwin", "arm64", "mac"),
        ("Darwin", "x86_64", "mac"),
        ("Linux", "x86_64", "linux"),
        ("Windows", "AMD64", "win"),
    ],
)
def test_cmdline_tools_url(mock_tools, android_sdk, host_os, host_arch, name):
    """Validate that the SDK URL is computed using `host_os`."""
    mock_tools.host_os = host_os
    mock_tools.host_arch = host_arch

    assert android_sdk.cmdline_tools_url == (
        f"https://dl.google.com/android/repository/commandlinetools-{name}-{SDK_MGR_DL_VER}_latest.zip"
    )
    assert_url_resolvable(android_sdk.cmdline_tools_url)


@pytest.mark.parametrize(
    "host_os, sdkmanager_name",
    [("Windows", "sdkmanager.bat"), ("NonWindows", "sdkmanager")],
)
def test_sdkmanager_path(mock_tools, android_sdk, host_os, sdkmanager_name):
    """Validate that if the user is on Windows, we run `sdkmanager.bat`, otherwise,
    `sdkmanager`."""
    # Mock out `host_os` so we can test Windows when not on Windows.
    mock_tools.host_os = host_os

    assert android_sdk.sdkmanager_path == (
        android_sdk.root_path / "cmdline-tools" / SDK_MGR_VER / "bin" / sdkmanager_name
    )


@pytest.mark.parametrize(
    "host_os, adb_name",
    [("Windows", "adb.exe"), ("NonWindows", "adb")],
)
def test_adb_path(mock_tools, android_sdk, host_os, adb_name):
    """Validate that if the user is on Windows, we run `adb.bat`, otherwise, `adb`."""
    # Mock out `host_os` so we can test Windows when not on Windows.
    mock_tools.host_os = host_os

    assert android_sdk.adb_path == (android_sdk.root_path / "platform-tools" / adb_name)


@pytest.mark.parametrize(
    "host_os, avdmanager_name",
    [("Windows", "avdmanager.bat"), ("NonWindows", "avdmanager")],
)
def test_avdmanager_path(mock_tools, android_sdk, host_os, avdmanager_name):
    """Validate that if the user is on Windows, we run `avdmanager.bat`, otherwise,
    `avdmanager`."""
    # Mock out `host_os` so we can test Windows when not on Windows.
    mock_tools.host_os = host_os

    assert android_sdk.avdmanager_path == (
        android_sdk.root_path / "cmdline-tools" / SDK_MGR_VER / "bin" / avdmanager_name
    )


@pytest.mark.parametrize(
    "host_os, emulator_name",
    [("Windows", "emulator.exe"), ("NonWindows", "emulator")],
)
def test_emulator_path(mock_tools, android_sdk, host_os, emulator_name):
    """Validate that if the user is on Windows, we run `emulator.bat`, otherwise,
    `emulator`."""
    # Mock out `host_os` so we can test Windows when not on Windows.
    mock_tools.host_os = host_os

    assert android_sdk.emulator_path == (
        android_sdk.root_path / "emulator" / emulator_name
    )


def test_avd_path(mock_tools, android_sdk, tmp_path):
    assert android_sdk.avd_path == tmp_path / "home/.android/avd"


def test_simple_env(mock_tools, android_sdk, tmp_path):
    """The SDK Environment can be constructed."""
    assert android_sdk.env == {
        "JAVA_HOME": os.fsdecode(Path("/path/to/jdk")),
        "ANDROID_HOME": os.fsdecode(tmp_path / "sdk"),
        "ANDROID_SDK_ROOT": os.fsdecode(tmp_path / "sdk"),
    }


def test_managed_install(mock_tools, android_sdk):
    """All Android SDK installs are managed."""
    assert android_sdk.managed_install


@pytest.mark.parametrize(
    "host_os, host_arch, emulator_abi",
    [
        ("Darwin", "x86_64", "x86_64"),
        ("Darwin", "arm64", "arm64-v8a"),
        ("Windows", "AMD64", "x86_64"),
        ("Linux", "x86_64", "x86_64"),
        ("Linux", "aarch64", "arm64-v8a"),
    ],
)
def test_emulator_abi(mock_tools, android_sdk, host_os, host_arch, emulator_abi):
    """The emulator API can be determined from the host OS and architecture."""
    # Mock the hardware and operating system
    mock_tools.host_os = host_os
    mock_tools.host_arch = host_arch

    assert android_sdk.emulator_abi == emulator_abi


@pytest.mark.parametrize(
    "host_os, host_arch",
    [
        ("Darwin", "powerpc"),
        ("Windows", "arm64"),
        ("Windows", "powerpc"),
        ("Linux", "arm64"),
        ("Linux", "powerpc"),
    ],
)
def test_bad_emulator_abi(mock_tools, android_sdk, host_os, host_arch):
    """If the host OS/architecture isn't supported by Android, an error is raised."""
    # Mock the hardware and operating system
    mock_tools.host_os = host_os
    mock_tools.host_arch = host_arch

    with pytest.raises(
        BriefcaseCommandError,
        match=rf"The Android emulator does not currently support {host_os} {host_arch} hardware.",
    ):
        _ = android_sdk.emulator_abi


def test_adb_for_device(mock_tools, android_sdk):
    """An ADB instance can be bound to a device."""
    adb = android_sdk.adb("some-device")

    assert adb.tools == mock_tools
    assert adb.device == "some-device"



import pytest

from briefcase.console import Console, Log
from briefcase.exceptions import UnsupportedHostError
from briefcase.platforms.macOS.xcode import macOSXcodeCreateCommand


@pytest.fixture
def create_command(tmp_path):
    return macOSXcodeCreateCommand(
        logger=Log(),
        console=Console(),
        base_path=tmp_path / "base_path",
        data_path=tmp_path / "briefcase",
    )


@pytest.mark.parametrize("host_os", ["Linux", "Windows", "WeirdOS"])
def test_unsupported_host_os(create_command, host_os):
    """Error raised for an unsupported OS."""
    create_command.tools.host_os = host_os

    with pytest.raises(
        UnsupportedHostError,
        match="macOS applications require the Xcode command line tools, which are only available on macOS.",
    ):
        create_command()


def test_binary_path(create_command, first_app_config, tmp_path):
    binary_path = create_command.binary_path(first_app_config)

    expected_path = (
        tmp_path
        / "base_path"
        / "build"
        / "first-app"
        / "macos"
        / "xcode"
        / "build"
        / "Release"
        / "First App.app"
    )
    assert binary_path == expected_path


def test_project_path(create_command, first_app_config, tmp_path):
    """The project path is the Xcode project."""
    project_path = create_command.project_path(first_app_config)

    expected_path = (
        tmp_path
        / "base_path"
        / "build"
        / "first-app"
        / "macos"
        / "xcode"
        / "First App.xcodeproj"
    )
    assert expected_path == project_path



__all__ = [
    "__version__",
]

try:
    # Read version from SCM metadata
    # This will only exist in a development environment
    from setuptools_scm import get_version

    # Excluded from coverage because a pure test environment (such as the one
    # used by tox in CI) won't have setuptools_scm
    __version__ = get_version("../..", relative_to=__file__)  # pragma: no cover
except (ModuleNotFoundError, LookupError):  # pragma: no-cover-if-missing-setuptools_scm
    # If setuptools_scm isn't in the environment, the call to import will fail.
    # If it *is* in the environment, but the code isn't a git checkout (e.g.,
    # it's been pip installed non-editable) the call to get_version() will fail.
    # If either of these occurs, read version from the installer metadata.
    from importlib.metadata import version

    __version__ = version("briefcase")



import pytest

from briefcase.integrations.windows_sdk import WindowsSDK


@pytest.fixture
def windows_sdk(mock_tools, tmp_path):
    return WindowsSDK(
        mock_tools,
        root_path=tmp_path,
        version="10.0.19000.0",
        arch="x64",
    )


def test_managed_install(windows_sdk):
    """All Windows SDK installs are unmanaged."""
    assert windows_sdk.managed_install is False


@pytest.mark.parametrize("sdk_arch", ["x64", "arm64", "gothic"])
def test_paths(windows_sdk, tmp_path, sdk_arch):
    """Windows SDK paths are appropriate for arch and version."""
    windows_sdk.arch = sdk_arch

    expected_bin_path = tmp_path / "bin/10.0.19000.0" / sdk_arch
    assert windows_sdk.bin_path == expected_bin_path
    assert windows_sdk.signtool_exe == expected_bin_path / "signtool.exe"



from briefcase.bootstraps.base import BaseGuiBootstrap


class PygameGuiBootstrap(BaseGuiBootstrap):
    display_name_annotation = "does not support iOS/Android/Web deployment"

    def app_source(self):
        return """\
import importlib.metadata
import os
import sys
from pathlib import Path

import pygame


SCREEN_WIDTH, SCREEN_HEIGHT = 800, 600
WHITE = (255, 255, 255)


def main():
    # Linux desktop environments use an app's .desktop file to integrate the app
    # in to their application menus. The .desktop file of this app will include
    # the StartupWMClass key, set to app's formal name. This helps associate the
    # app's windows to its menu item.
    #
    # For association to work, any windows of the app must have WMCLASS property
    # set to match the value set in app's desktop file. For PyGame, this is set
    # using the SDL_VIDEO_X11_WMCLASS environment variable.

    # Find the name of the module that was used to start the app
    app_module = sys.modules["__main__"].__package__
    # Retrieve the app's metadata
    metadata = importlib.metadata.metadata(app_module)

    os.environ["SDL_VIDEO_X11_WMCLASS"] = metadata["Formal-Name"]

    pygame.init()
    pygame.display.set_caption(metadata["Formal-Name"])
    screen = pygame.display.set_mode((SCREEN_WIDTH, SCREEN_HEIGHT))

    running = True
    while running:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                running = False
                break

        screen.fill(WHITE)
        pygame.display.flip()

    pygame.quit()
"""

    def pyproject_table_briefcase_app_extra_content(self):
        return """
requires = [
    "pygame~=2.2",
]
test_requires = [
{% if cookiecutter.test_framework == "pytest" %}
    "pytest",
{% endif %}
]
"""

    def pyproject_table_macOS(self):
        return """\
universal_build = true
requires = [
    "std-nslog~=1.0.0",
]
"""

    def pyproject_table_linux(self):
        return """\
requires = [
]
"""

    def pyproject_table_linux_system_debian(self):
        return """\
system_requires = [
]

system_runtime_requires = [
]
"""

    def pyproject_table_linux_system_rhel(self):
        return """\
system_requires = [
]

system_runtime_requires = [
]
"""

    def pyproject_table_linux_system_suse(self):
        return """\
system_requires = [
]

system_runtime_requires = [
]
"""

    def pyproject_table_linux_system_arch(self):
        return """\
system_requires = [
]

system_runtime_requires = [
]
"""

    def pyproject_table_linux_appimage(self):
        return """\
manylinux = "manylinux_2_28"

system_requires = [
]

linuxdeploy_plugins = [
]
"""

    def pyproject_table_linux_flatpak(self):
        return """\
flatpak_runtime = "org.freedesktop.Platform"
flatpak_runtime_version = "23.08"
flatpak_sdk = "org.freedesktop.Sdk"
"""

    def pyproject_table_windows(self):
        return """\
requires = [
]
"""

    def pyproject_table_iOS(self):
        return """\
supported = false
"""

    def pyproject_table_android(self):
        return """\
supported = false
"""

    def pyproject_table_web(self):
        return """\
supported = false
"""



import os
from pathlib import Path
from unittest.mock import ANY

import pytest

from briefcase.console import LogLevel

from .conftest import CREATE_NEW_PROCESS_GROUP, CREATE_NO_WINDOW


@pytest.mark.parametrize("platform", ["Linux", "Darwin", "Windows"])
def test_call(mock_sub, capsys, platform, sub_kw):
    """A simple call will be invoked."""

    mock_sub.tools.host_os = platform
    mock_sub.Popen(["hello", "world"])

    mock_sub._subprocess.Popen.assert_called_with(["hello", "world"], **sub_kw)
    assert capsys.readouterr().out == ""


def test_call_with_arg(mock_sub, capsys, sub_kw):
    """Any extra keyword arguments are passed through as-is."""

    mock_sub.Popen(["hello", "world"], extra_kw="extra")

    mock_sub._subprocess.Popen.assert_called_with(
        ["hello", "world"],
        extra_kw="extra",
        **sub_kw,
    )
    assert capsys.readouterr().out == ""


def test_call_with_path_arg(mock_sub, capsys, tmp_path, sub_kw):
    """Path-based arguments are converted to strings and passed in as-is."""

    mock_sub.Popen(["hello", tmp_path / "location"], cwd=tmp_path / "cwd")

    mock_sub._subprocess.Popen.assert_called_with(
        ["hello", os.fsdecode(tmp_path / "location")],
        cwd=os.fsdecode(tmp_path / "cwd"),
        **sub_kw,
    )
    assert capsys.readouterr().out == ""


@pytest.mark.parametrize(
    ("platform", "start_new_session", "popen_kwargs"),
    [
        ("Linux", None, {}),
        ("Linux", True, {}),
        ("Linux", False, {}),
        ("Darwin", None, {}),
        ("Darwin", True, {}),
        ("Darwin", False, {}),
        ("Windows", None, {}),
        (
            "Windows",
            True,
            {"creationflags": CREATE_NEW_PROCESS_GROUP | CREATE_NO_WINDOW},
        ),
        ("Windows", False, {}),
    ],
)
def test_call_with_start_new_session(
    mock_sub,
    capsys,
    platform,
    start_new_session,
    popen_kwargs,
    sub_kw,
):
    """start_new_session is passed thru on Linux and macOS but converted for Windows."""

    mock_sub.tools.host_os = platform
    mock_sub.Popen(["hello", "world"], start_new_session=start_new_session)

    final_kwargs = {**sub_kw, **popen_kwargs}
    if platform == "Windows":
        mock_sub._subprocess.Popen.assert_called_with(
            ["hello", "world"], **final_kwargs
        )
        assert capsys.readouterr().out == ""
    else:
        mock_sub._subprocess.Popen.assert_called_with(
            ["hello", "world"],
            start_new_session=start_new_session,
            **final_kwargs,
        )
        assert capsys.readouterr().out == ""


@pytest.mark.parametrize(
    ("creationflags", "final_creationflags"),
    [
        (0x1, CREATE_NEW_PROCESS_GROUP | CREATE_NO_WINDOW | 1),
        (CREATE_NEW_PROCESS_GROUP, CREATE_NEW_PROCESS_GROUP | CREATE_NO_WINDOW),
        (0, CREATE_NEW_PROCESS_GROUP | CREATE_NO_WINDOW),
    ],
)
def test_call_windows_with_start_new_session_and_creationflags(
    mock_sub,
    capsys,
    creationflags,
    final_creationflags,
):
    """Creationflags used to simulate start_new_session=True should be merged with any
    existing flags."""

    mock_sub.tools.host_os = "Windows"

    # use commented test below when merging creationflags is allowed
    with pytest.raises(
        AssertionError, match="Subprocess called with creationflags set"
    ):
        mock_sub.Popen(
            ["hello", "world"],
            start_new_session=True,
            creationflags=creationflags,
        )


def test_debug_call(mock_sub, capsys, sub_kw):
    """If verbosity is turned up, there is output."""
    mock_sub.tools.logger.verbosity = LogLevel.DEBUG
    mock_sub.Popen(["hello", "world"])

    mock_sub._subprocess.Popen.assert_called_with(["hello", "world"], **sub_kw)

    assert capsys.readouterr().out == (
        "\n"
        ">>> Running Command:\n"
        ">>>     hello world\n"
        ">>> Working Directory:\n"
        f">>>     {Path.cwd()}\n"
    )


def test_debug_call_with_env(mock_sub, capsys, tmp_path, sub_kw):
    """If verbosity is turned up, and injected env vars are included output."""
    mock_sub.tools.logger.verbosity = LogLevel.DEBUG

    env = {"NewVar": "NewVarValue"}
    mock_sub.Popen(["hello", "world"], env=env, cwd=tmp_path / "cwd")

    merged_env = mock_sub.tools.os.environ.copy()
    merged_env.update(env)

    mock_sub._subprocess.Popen.assert_called_with(
        ["hello", "world"],
        env=merged_env,
        cwd=os.fsdecode(tmp_path / "cwd"),
        **sub_kw,
    )

    expected_output = (
        "\n"
        ">>> Running Command:\n"
        ">>>     hello world\n"
        ">>> Working Directory:\n"
        f">>>     {tmp_path / 'cwd'}\n"
        ">>> Environment Overrides:\n"
        ">>>     NewVar=NewVarValue\n"
    )
    assert capsys.readouterr().out == expected_output


@pytest.mark.parametrize(
    "in_kwargs, kwargs",
    [
        ({}, {"text": True, "encoding": ANY, "errors": "backslashreplace"}),
        ({"text": True}, {"text": True, "encoding": ANY, "errors": "backslashreplace"}),
        ({"text": False}, {"text": False}),
        ({"universal_newlines": False}, {"text": False}),
        (
            {"universal_newlines": True},
            {"text": True, "encoding": ANY, "errors": "backslashreplace"},
        ),
    ],
)
def test_text_eq_true_default_overriding(mock_sub, in_kwargs, kwargs):
    """If text or universal_newlines is explicitly provided, those should override
    text=true default and universal_newlines should be converted to text."""

    mock_sub.Popen(["hello", "world"], **in_kwargs)
    mock_sub._subprocess.Popen.assert_called_with(["hello", "world"], **kwargs)



import re
import shutil
import subprocess
from pathlib import Path
from unittest import mock

import pytest
import tomli_w

import briefcase.platforms.windows.app
from briefcase.console import Console, Log
from briefcase.exceptions import BriefcaseCommandError
from briefcase.integrations.rcedit import RCEdit
from briefcase.integrations.subprocess import Subprocess
from briefcase.integrations.windows_sdk import WindowsSDK
from briefcase.platforms.windows.app import WindowsAppBuildCommand

from ....utils import create_file


@pytest.fixture
def build_command(tmp_path):
    command = WindowsAppBuildCommand(
        logger=Log(),
        console=Console(),
        base_path=tmp_path / "base_path",
        data_path=tmp_path / "briefcase",
    )
    command.tools.host_os = "Windows"
    command.tools.host_arch = "AMD64"
    command.tools.subprocess = mock.MagicMock(spec_set=Subprocess)
    command.tools.shutil = mock.MagicMock(spec_set=shutil)
    command.tools.file.download = mock.MagicMock()
    command.tools.rcedit = RCEdit(command.tools)
    return command


@pytest.fixture
def windows_sdk(build_command, tmp_path):
    return WindowsSDK(
        tools=build_command.tools,
        root_path=tmp_path / "win_sdk",
        version="86.1.1",
        arch="x64",
    )


def test_verify_without_windows_sdk(build_command, monkeypatch):
    """Verifying on Windows creates an RCEdit wrapper."""
    mock_sdk = mock.MagicMock(spec_set=WindowsSDK)
    monkeypatch.setattr(briefcase.platforms.windows.app, "WindowsSDK", mock_sdk)
    mock_sdk.verify.side_effect = BriefcaseCommandError("Windows SDK")

    mock_rcedit_verify = mock.MagicMock(wraps=RCEdit.verify)
    monkeypatch.setattr(
        briefcase.platforms.windows.app.RCEdit,
        "verify",
        mock_rcedit_verify,
    )

    build_command.verify_tools()

    # RCEdit tool was verified
    mock_rcedit_verify.assert_called_once_with(tools=build_command.tools)
    assert isinstance(build_command.tools.rcedit, RCEdit)
    # Windows SDK tool not created
    assert not hasattr(build_command.tools, "windows_sdk")


def test_verify_with_windows_sdk(build_command, windows_sdk, monkeypatch):
    """Verifying on Windows creates an RCEdit and Windows SDK wrapper."""
    build_command.tools.windows_sdk = windows_sdk

    mock_windows_sdk_verify = mock.MagicMock(wraps=WindowsSDK.verify)
    monkeypatch.setattr(
        briefcase.platforms.windows.app.WindowsSDK,
        "verify",
        mock_windows_sdk_verify,
    )

    mock_rcedit_verify = mock.MagicMock(wraps=RCEdit.verify)
    monkeypatch.setattr(
        briefcase.platforms.windows.app.RCEdit,
        "verify",
        mock_rcedit_verify,
    )

    build_command.verify_tools()

    # RCEdit tool was verified
    mock_rcedit_verify.assert_called_once_with(tools=build_command.tools)
    assert isinstance(build_command.tools.rcedit, RCEdit)
    # WindowsSDK tool was verified
    mock_windows_sdk_verify.assert_called_once_with(tools=build_command.tools)
    assert isinstance(build_command.tools.windows_sdk, WindowsSDK)


@pytest.mark.parametrize("pre_existing", [True, False])
@pytest.mark.parametrize("console_app", [True, False])
def test_build_app_without_windows_sdk(
    build_command,
    first_app_templated,
    pre_existing,
    console_app,
    tmp_path,
):
    """The stub binary will be updated when a Windows app is built."""
    first_app_templated.console_app = console_app

    exec_path = tmp_path / "base_path/build/first-app/windows/app/src"
    if pre_existing:
        # If this is a pre-existing app, the stub has already been renamed
        if console_app:
            (exec_path / "Stub.exe").rename(exec_path / "first-app.exe")
        else:
            (exec_path / "Stub.exe").rename(exec_path / "First App.exe")

    build_command.build_app(first_app_templated)

    # The stub binary has been renamed
    assert not (exec_path / "Stub.exe").is_file()
    if console_app:
        assert (exec_path / "first-app.exe").is_file()
    else:
        assert (exec_path / "First App.exe").is_file()

    # update the app binary resources
    build_command.tools.subprocess.run.assert_called_once_with(
        [
            tmp_path / "briefcase/tools/rcedit-x64.exe",
            Path("src/first-app.exe") if console_app else Path("src/First App.exe"),
            "--set-version-string",
            "CompanyName",
            "Megacorp",
            "--set-version-string",
            "FileDescription",
            "First App",
            "--set-version-string",
            "FileVersion",
            "0.0.1",
            "--set-version-string",
            "InternalName",
            "first_app",
            "--set-version-string",
            "OriginalFilename",
            "first-app.exe" if console_app else "First App.exe",
            "--set-version-string",
            "ProductName",
            "First App",
            "--set-version-string",
            "ProductVersion",
            "0.0.1",
            "--set-icon",
            "icon.ico",
        ],
        check=True,
        cwd=tmp_path / "base_path/build/first-app/windows/app",
    )


@pytest.mark.parametrize("console_app", [True, False])
def test_build_app_with_windows_sdk(
    build_command,
    windows_sdk,
    first_app_templated,
    console_app,
    tmp_path,
):
    """The stub binary will be updated when a Windows app is built."""
    build_command.tools.windows_sdk = windows_sdk
    first_app_templated.console_app = console_app

    build_command.build_app(first_app_templated)

    # remove any digital signatures on the app binary
    build_command.tools.subprocess.check_output.assert_called_once_with(
        [
            tmp_path / "win_sdk/bin/86.1.1/x64/signtool.exe",
            "remove",
            "-s",
            Path("src/first-app.exe") if console_app else Path("src/First App.exe"),
        ],
        cwd=tmp_path / "base_path/build/first-app/windows/app",
    )
    # update the app binary resources
    build_command.tools.subprocess.run.assert_called_once_with(
        [
            tmp_path / "briefcase/tools/rcedit-x64.exe",
            Path("src/first-app.exe") if console_app else Path("src/First App.exe"),
            "--set-version-string",
            "CompanyName",
            "Megacorp",
            "--set-version-string",
            "FileDescription",
            "First App",
            "--set-version-string",
            "FileVersion",
            "0.0.1",
            "--set-version-string",
            "InternalName",
            "first_app",
            "--set-version-string",
            "OriginalFilename",
            "first-app.exe" if console_app else "First App.exe",
            "--set-version-string",
            "ProductName",
            "First App",
            "--set-version-string",
            "ProductVersion",
            "0.0.1",
            "--set-icon",
            "icon.ico",
        ],
        check=True,
        cwd=tmp_path / "base_path/build/first-app/windows/app",
    )


def test_build_app_without_any_digital_signatures(
    build_command,
    windows_sdk,
    first_app_templated,
    tmp_path,
):
    """If the app binary is not already signed, then attempt to remove signatures fails
    but app build succeeds."""
    build_command.tools.windows_sdk = windows_sdk

    build_command.tools.subprocess.check_output.side_effect = subprocess.CalledProcessError(
        returncode=1,
        cmd="signtool.exe remove -s app.exe",
        output="""
    Number of errors: 1
    SignTool Error: CryptSIPRemoveSignedDataMsg returned error: 0x00000057
            The parameter is incorrect.
""",
    )

    build_command.build_app(first_app_templated)

    # remove any digital signatures on the app binary
    build_command.tools.subprocess.check_output.assert_called_once_with(
        [
            tmp_path / "win_sdk/bin/86.1.1/x64/signtool.exe",
            "remove",
            "-s",
            Path("src/First App.exe"),
        ],
        cwd=tmp_path / "base_path/build/first-app/windows/app",
    )
    # update the app binary resources
    build_command.tools.subprocess.run.assert_called_once_with(
        [
            tmp_path / "briefcase/tools/rcedit-x64.exe",
            Path("src/First App.exe"),
            "--set-version-string",
            "CompanyName",
            "Megacorp",
            "--set-version-string",
            "FileDescription",
            "First App",
            "--set-version-string",
            "FileVersion",
            "0.0.1",
            "--set-version-string",
            "InternalName",
            "first_app",
            "--set-version-string",
            "OriginalFilename",
            "First App.exe",
            "--set-version-string",
            "ProductName",
            "First App",
            "--set-version-string",
            "ProductVersion",
            "0.0.1",
            "--set-icon",
            "icon.ico",
        ],
        check=True,
        cwd=tmp_path / "base_path/build/first-app/windows/app",
    )


def test_build_app_error_remove_signature(
    build_command,
    windows_sdk,
    first_app_templated,
    tmp_path,
):
    """If the attempt to remove any exist digital signatures fails because signtool
    raises an unexpected error, then the build fails."""
    build_command.tools.windows_sdk = windows_sdk

    build_command.tools.subprocess.check_output.side_effect = subprocess.CalledProcessError(
        returncode=1,
        cmd="signtool.exe remove /s filepath",
        output="""
    Number of errors: 1
    Unknown and unexpected error
""",
    )

    error_message = (
        "Failed to remove any existing digital signatures from the stub app.\n"
        "\n"
        "Recreating the app layout may also help resolve this issue:\n"
        "\n"
        "    $ briefcase create windows app\n"
        "\n"
    )
    with pytest.raises(BriefcaseCommandError, match=re.escape(error_message)):
        build_command.build_app(first_app_templated)

    # remove any digital signatures on the app binary
    build_command.tools.subprocess.check_output.assert_called_once_with(
        [
            tmp_path / "win_sdk/bin/86.1.1/x64/signtool.exe",
            "remove",
            "-s",
            Path("src/First App.exe"),
        ],
        cwd=tmp_path / "base_path/build/first-app/windows/app",
    )
    # update the app binary resources not called
    build_command.tools.subprocess.run.assert_not_called()


def test_build_app_failure(build_command, first_app_templated):
    """If the stub binary cannot be updated, an error is raised."""

    build_command.tools.subprocess.run.side_effect = subprocess.CalledProcessError(
        returncode=1,
        cmd="rcedit-x64.exe",
    )

    with pytest.raises(
        BriefcaseCommandError,
        match=r"Unable to update details on stub app for first-app.",
    ):
        build_command.build_app(first_app_templated)


def test_build_app_with_support_package_update(
    build_command,
    first_app_templated,
    tmp_path,
    windows_sdk,
    capsys,
):
    """If a support package update is performed, the user is warned."""

    # To trigger the app package update logic, we need to invoke the full build
    # command, and fake being on a verified Windows install with a generated
    # app.
    build_command.tools.host_os = "Windows"
    build_command.tools.windows_sdk = windows_sdk

    # Hard code a support revision so that the download support package is fixed
    first_app_templated.support_revision = "1"

    # Fake the existence of some source files.
    create_file(
        tmp_path / "base_path/src/first_app/app.py",
        "print('an app')",
    )

    # Populate a briefcase.toml that mirrors a real Windows app
    with (build_command.bundle_path(first_app_templated) / "briefcase.toml").open(
        "wb"
    ) as f:
        index = {
            "paths": {
                "app_path": "src/app",
                "app_package_path": "src/app_packages",
                "support_path": "src",
            }
        }
        tomli_w.dump(index, f)

    # Build the app with a support package update
    build_command(first_app_templated, update_support=True)

    # update the app binary resources
    build_command.tools.subprocess.run.assert_called_once_with(
        [
            tmp_path / "briefcase/tools/rcedit-x64.exe",
            Path("src/First App.exe"),
            "--set-version-string",
            "CompanyName",
            "Megacorp",
            "--set-version-string",
            "FileDescription",
            "First App",
            "--set-version-string",
            "FileVersion",
            "0.0.1",
            "--set-version-string",
            "InternalName",
            "first_app",
            "--set-version-string",
            "OriginalFilename",
            "First App.exe",
            "--set-version-string",
            "ProductName",
            "First App",
            "--set-version-string",
            "ProductVersion",
            "0.0.1",
            "--set-icon",
            "icon.ico",
        ],
        check=True,
        cwd=tmp_path / "base_path/build/first-app/windows/app",
    )

    # No attempt was made to clean up the support package.
    build_command.tools.shutil.rmtree.assert_not_called()

    # The user was warned that support package update may not work.
    assert "WARNING: Support package update may be imperfect" in capsys.readouterr().out



import os
import shutil
import sys
from unittest import mock

import pytest
from requests import exceptions as requests_exceptions

from briefcase.exceptions import (
    InvalidSupportPackage,
    MissingNetworkResourceError,
    MissingSupportPackage,
    NetworkFailure,
)

from ...utils import (
    create_zip_file,
    mock_file_download,
    mock_tgz_download,
    mock_zip_download,
)


def test_install_app_support_package(
    create_command,
    myapp,
    tmp_path,
    support_path,
    app_requirements_path_index,
):
    """A support package can be downloaded and unpacked where it is needed."""
    # Mock download.file to return a support package
    create_command.tools.file.download = mock.MagicMock(
        side_effect=mock_tgz_download(
            "Python-3.X-tester-support.b37.tar.gz",
            [("internal/file.txt", "hello world")],
        )
    )

    # Mock shutil so we can confirm that unpack is called,
    # but we still want the side effect of calling it
    create_command.tools.shutil = mock.MagicMock(spec_set=shutil)
    create_command.tools.shutil.unpack_archive.side_effect = shutil.unpack_archive

    # Install the support package
    create_command.install_app_support_package(myapp)

    # Confirm the right URL was used
    create_command.tools.file.download.assert_called_with(
        download_path=create_command.data_path / "support",
        url="https://briefcase-support.s3.amazonaws.com/python/3.X/Tester/Python-3.X-Tester-support.b37.tar.gz",
        role="support package",
    )

    # Confirm the right file was unpacked
    create_command.tools.shutil.unpack_archive.assert_called_with(
        filename=tmp_path / "data/support/Python-3.X-tester-support.b37.tar.gz",
        extract_dir=support_path,
        **({"filter": "data"} if sys.version_info >= (3, 12) else {}),
    )

    # Confirm that the full path to the support file
    # has been unpacked.
    assert (support_path / "internal/file.txt").exists()


def test_install_pinned_app_support_package(
    create_command,
    myapp,
    tmp_path,
    support_path,
    app_requirements_path_index,
):
    """A pinned support package can be downloaded and unpacked where it is needed."""
    # Pin the support revision
    myapp.support_revision = "42"

    # Mock download.file to return a support package
    create_command.tools.file.download = mock.MagicMock(
        side_effect=mock_tgz_download(
            "Python-3.X-Tester-support.b42.tar.gz",
            [("internal/file.txt", "hello world")],
        )
    )

    # Mock shutil so we can confirm that unpack is called,
    # but we still want the side effect of calling it
    create_command.tools.shutil = mock.MagicMock(spec_set=shutil)
    create_command.tools.shutil.unpack_archive.side_effect = shutil.unpack_archive

    # Install the support package
    create_command.install_app_support_package(myapp)

    # Confirm the right URL was used
    create_command.tools.file.download.assert_called_with(
        download_path=create_command.data_path / "support",
        url="https://briefcase-support.s3.amazonaws.com/python/3.X/Tester/Python-3.X-Tester-support.b42.tar.gz",
        role="support package",
    )

    # Confirm the right file was unpacked
    create_command.tools.shutil.unpack_archive.assert_called_with(
        filename=tmp_path / "data/support/Python-3.X-Tester-support.b42.tar.gz",
        extract_dir=support_path,
        **({"filter": "data"} if sys.version_info >= (3, 12) else {}),
    )

    # Confirm that the full path to the support file
    # has been unpacked.
    assert (support_path / "internal/file.txt").exists()


def test_install_custom_app_support_package_file(
    create_command,
    myapp,
    tmp_path,
    support_path,
    app_requirements_path_index,
):
    """A custom support package can be specified as a local file."""
    # Provide an app-specific override of the package URL
    myapp.support_package = os.fsdecode(tmp_path / "custom/support.zip")

    # Write a temporary support zip file
    support_file = create_zip_file(
        tmp_path / "custom/support.zip",
        [("internal/file.txt", "hello world")],
    )

    # Modify download.file to return the temp zipfile
    create_command.tools.file.download = mock.MagicMock()

    # Mock shutil so we can confirm that unpack is called,
    # but we still want the side effect of calling it
    create_command.tools.shutil = mock.MagicMock(spec_set=shutil)
    create_command.tools.shutil.unpack_archive.side_effect = shutil.unpack_archive

    # Install the support package
    create_command.install_app_support_package(myapp)

    # There should have been no download attempt,
    # as the resource is local.
    create_command.tools.file.download.assert_not_called()

    # Confirm the right file was unpacked
    create_command.tools.shutil.unpack_archive.assert_called_with(
        filename=support_file,
        extract_dir=support_path,
    )

    # Confirm that the full path to the support file
    # has been unpacked.
    assert (support_path / "internal/file.txt").exists()


def test_install_custom_app_support_package_file_with_revision(
    create_command,
    myapp,
    tmp_path,
    support_path,
    app_requirements_path_index,
    capsys,
):
    """If a custom support package file also specifies a revision, the revision is
    ignored with a warning."""
    # Provide an app-specific override of the package URL
    myapp.support_package = os.fsdecode(tmp_path / "custom/support.zip")
    myapp.support_revision = "42"

    # Write a temporary support zip file
    support_file = create_zip_file(
        tmp_path / "custom/support.zip",
        [("internal/file.txt", "hello world")],
    )

    # Modify download.file to return the temp zipfile
    create_command.tools.file.download = mock.MagicMock()

    # Mock shutil so we can confirm that unpack is called,
    # but we still want the side effect of calling it
    create_command.tools.shutil = mock.MagicMock()
    create_command.tools.shutil.unpack_archive.side_effect = shutil.unpack_archive

    # Install the support package
    create_command.install_app_support_package(myapp)

    # There should have been no download attempt,
    # as the resource is local.
    create_command.tools.file.download.assert_not_called()

    # Confirm the right file was unpacked
    create_command.tools.shutil.unpack_archive.assert_called_with(
        filename=support_file,
        extract_dir=support_path,
    )

    # Confirm that the full path to the support file
    # has been unpacked.
    assert (support_path / "internal/file.txt").exists()

    # A warning about the support revision was generated.
    assert "support revision will be ignored." in capsys.readouterr().out


def test_support_package_url_with_invalid_custom_support_package_url(
    create_command,
    myapp,
    app_requirements_path_index,
):
    """Invalid URL for a custom support package raises MissingNetworkResourceError."""

    # Provide an custom support URL
    url = "https://example.com/custom/support.zip"
    myapp.support_package = url

    # Modify download.file to raise an exception
    create_command.tools.file.download = mock.MagicMock(
        side_effect=MissingNetworkResourceError(url)
    )

    # The bad URL should raise a MissingNetworkResourceError
    with pytest.raises(MissingNetworkResourceError):
        create_command.install_app_support_package(myapp)

    # However, there will have been a download attempt
    create_command.tools.file.download.assert_called_with(
        download_path=(
            create_command.data_path
            / "support"
            / "55441abbffa311f65622df45a943afc347a21ab40e8dcec79472c92ef467db24"
        ),
        url=url,
        role="support package",
    )


def test_support_package_url_with_unsupported_platform(
    create_command,
    myapp,
    app_requirements_path_index,
):
    """An unsupported platform raises MissingSupportPackage."""
    # Modify download.file to raise an exception due to missing support package
    create_command.tools.file.download = mock.MagicMock(
        side_effect=MissingNetworkResourceError(
            url="https://briefcase-support.s3.amazonaws.com/python/3.X/Tester/Python-3.X-Tester-support.b37.tar.gz",
        )
    )

    # The unknown platform should cause a missing support package error
    with pytest.raises(MissingSupportPackage):
        create_command.install_app_support_package(myapp)

    # However, there will have been a download attempt
    create_command.tools.file.download.assert_called_with(
        download_path=create_command.data_path / "support",
        url="https://briefcase-support.s3.amazonaws.com/python/3.X/Tester/Python-3.X-Tester-support.b37.tar.gz",
        role="support package",
    )


def test_install_custom_app_support_package_url(
    create_command,
    myapp,
    tmp_path,
    support_path,
    app_requirements_path_index,
):
    """A custom support package can be specified as URL."""
    # Provide an app-specific override of the package URL
    myapp.support_package = "https://example.com/custom/custom-support.zip"

    # Mock download.file to return a support package
    create_command.tools.file.download = mock.MagicMock(
        side_effect=mock_zip_download(
            "custom-support.zip",
            [("internal/file.txt", "hello world")],
        )
    )

    # Mock shutil so we can confirm that unpack is called,
    # but we still want the side effect of calling it
    create_command.tools.shutil = mock.MagicMock(spec_set=shutil)
    create_command.tools.shutil.unpack_archive.side_effect = shutil.unpack_archive

    # Install the support package
    create_command.install_app_support_package(myapp)

    # Confirm the right URL and download path was used
    create_command.tools.file.download.assert_called_with(
        download_path=(
            create_command.data_path
            / "support"
            / "1d3ac0e09eb22abc63c4e7b699b6ab5d58e277015eeae61070e3f9f11512e6b3"
        ),
        url="https://example.com/custom/custom-support.zip",
        role="support package",
    )

    # Confirm the right file was unpacked into the hashed location
    create_command.tools.shutil.unpack_archive.assert_called_with(
        filename=tmp_path
        / "data"
        / "support"
        / "1d3ac0e09eb22abc63c4e7b699b6ab5d58e277015eeae61070e3f9f11512e6b3"
        / "custom-support.zip",
        extract_dir=support_path,
    )

    # Confirm that the full path to the support file
    # has been unpacked.
    assert (support_path / "internal/file.txt").exists()


def test_install_custom_app_support_package_url_with_revision(
    create_command,
    myapp,
    tmp_path,
    support_path,
    app_requirements_path_index,
    capsys,
):
    """If a custom support package URL also specifies a revision, the revision is
    ignored with a warning."""
    # Provide an app-specific override of the package URL and revision
    myapp.support_package = "https://example.com/custom/custom-support.zip"
    myapp.support_revision = "42"

    # Mock download.file to return a support package
    create_command.tools.file.download = mock.MagicMock(
        side_effect=mock_zip_download(
            "custom-support.zip",
            [("internal/file.txt", "hello world")],
        )
    )

    # Mock shutil so we can confirm that unpack is called,
    # but we still want the side effect of calling it
    create_command.tools.shutil = mock.MagicMock(spec_set=shutil)
    create_command.tools.shutil.unpack_archive.side_effect = shutil.unpack_archive

    # Install the support package
    create_command.install_app_support_package(myapp)

    # Confirm the right URL and download path was used
    create_command.tools.file.download.assert_called_with(
        download_path=(
            create_command.data_path
            / "support"
            / "1d3ac0e09eb22abc63c4e7b699b6ab5d58e277015eeae61070e3f9f11512e6b3"
        ),
        url="https://example.com/custom/custom-support.zip",
        role="support package",
    )

    # Confirm the right file was unpacked into the hashed location
    create_command.tools.shutil.unpack_archive.assert_called_with(
        filename=tmp_path
        / "data"
        / "support"
        / "1d3ac0e09eb22abc63c4e7b699b6ab5d58e277015eeae61070e3f9f11512e6b3"
        / "custom-support.zip",
        extract_dir=support_path,
    )

    # Confirm that the full path to the support file
    # has been unpacked.
    assert (support_path / "internal/file.txt").exists()

    # A warning about the support revision was generated.
    assert "support revision will be ignored." in capsys.readouterr().out


def test_install_custom_app_support_package_url_with_args(
    create_command,
    myapp,
    tmp_path,
    support_path,
    app_requirements_path_index,
):
    """A custom support package can be specified as URL with args."""
    # Provide an app-specific override of the package URL
    myapp.support_package = "https://example.com/custom/custom-support.zip?cool=Yes"

    # Mock download.file to return a support package
    create_command.tools.file.download = mock.MagicMock(
        side_effect=mock_zip_download(
            "custom-support.zip",
            [("internal/file.txt", "hello world")],
        )
    )
    # Mock shutil so we can confirm that unpack is called,
    # but we still want the side effect of calling it
    create_command.tools.shutil = mock.MagicMock(spec_set=shutil)
    create_command.tools.shutil.unpack_archive.side_effect = shutil.unpack_archive

    # Install the support package
    create_command.install_app_support_package(myapp)

    # Confirm the right URL was used
    create_command.tools.file.download.assert_called_with(
        download_path=create_command.data_path
        / "support"
        / "f8cf64ad2ba249a1efbb63db60ebdc64f043035fbdd81934c6ad1e84a030c429",
        url="https://example.com/custom/custom-support.zip?cool=Yes",
        role="support package",
    )

    # Confirm the right file was unpacked
    create_command.tools.shutil.unpack_archive.assert_called_with(
        filename=tmp_path
        / "data"
        / "support"
        / "f8cf64ad2ba249a1efbb63db60ebdc64f043035fbdd81934c6ad1e84a030c429"
        / "custom-support.zip",
        extract_dir=support_path,
    )

    # Confirm that the full path to the support file
    # has been unpacked.
    assert (support_path / "internal/file.txt").exists()


def test_offline_install(
    create_command,
    myapp,
    support_path,
    app_requirements_path_index,
):
    """If the computer is offline, an error is raised."""
    create_command.tools.requests.get = mock.MagicMock(
        side_effect=requests_exceptions.ConnectionError
    )

    # Installing while offline raises an error
    with pytest.raises(NetworkFailure):
        create_command.install_app_support_package(myapp)


def test_invalid_support_package(
    create_command,
    myapp,
    tmp_path,
    support_path,
    app_requirements_path_index,
):
    """If the support package isn't a valid zipfile, an error is raised."""
    # Mock download.file to return a non-zip file
    create_command.tools.file.download = mock.MagicMock(
        side_effect=mock_file_download(
            "not-a.zip",
            "This isn't a zip file",
        )
    )

    # Installing the bad support package raises an error
    with pytest.raises(InvalidSupportPackage):
        create_command.install_app_support_package(myapp)


def test_missing_support_package(
    create_command,
    myapp,
    tmp_path,
    support_path,
    app_requirements_path_index,
):
    """If the path provided for the support package is bad, an error is raised."""
    # Set a custom support package that doesn't exist
    myapp.support_package = "/path/does/not/exist.zip"

    # Installing the bad support package raises an error
    with pytest.raises(InvalidSupportPackage):
        create_command.install_app_support_package(myapp)


def test_no_support_path(create_command, myapp, no_support_path_index):
    """If support_path is not listed in briefcase.toml, a support package will not be
    downloaded."""
    create_command.tools.file.download = mock.MagicMock()
    create_command.install_app_support_package(myapp)
    create_command.tools.file.download.assert_not_called()


def test_no_support_revision(create_command, myapp, no_support_revision_index):
    """If support_revision is not listed in briefcase.toml, a support package will not
    be downloaded."""
    create_command.tools.file.download = mock.MagicMock()

    # An error is raised when attempting to install the support package
    with pytest.raises(MissingSupportPackage):
        create_command.install_app_support_package(myapp)

    # No download attempt is made.
    create_command.tools.file.download.assert_not_called()



import pytest

from briefcase.config import is_valid_bundle_identifier


@pytest.mark.parametrize(
    "bundle",
    [
        "com.example",
        "com.example.more",
        "com.example42.more",
        "com.example-42.more",
        "in.example",  # Valid identifier with a country code as the TLD
        "is.example",  # Valid identifier with a country code as the TLD
        "com.example.in",  # Country codes can appear anywhere.
    ],
)
def test_valid_bundle(bundle):
    """Test that valid bundles are accepted."""
    assert is_valid_bundle_identifier(bundle)


@pytest.mark.parametrize(
    "bundle",
    [
        "not a bundle!",  # Free text.
        "home",  # Only one section.
        "com.hello_world",  # underscore
        "com.hello,world",  # comma
        "com.hello world!",  # exclamation point
    ],
)
def test_invalid_bundle(bundle):
    """Test that invalid bundles are rejected."""
    assert not is_valid_bundle_identifier(bundle)



from briefcase.bootstraps.base import BaseGuiBootstrap


class PySide6GuiBootstrap(BaseGuiBootstrap):
    display_name_annotation = "does not support iOS/Android/Web deployment"

    def app_source(self):
        return """\
import importlib.metadata
import sys

from PySide6 import QtWidgets


class {{ cookiecutter.class_name }}(QtWidgets.QMainWindow):
    def __init__(self):
        super().__init__()
        self.init_ui()

    def init_ui(self):
        self.setWindowTitle("{{ cookiecutter.app_name }}")
        self.show()


def main():
    # Linux desktop environments use an app's .desktop file to integrate the app
    # in to their application menus. The .desktop file of this app will include
    # the StartupWMClass key, set to app's formal name. This helps associate the
    # app's windows to its menu item.
    #
    # For association to work, any windows of the app must have WMCLASS property
    # set to match the value set in app's desktop file. For PySide6, this is set
    # with setApplicationName().

    # Find the name of the module that was used to start the app
    app_module = sys.modules["__main__"].__package__
    # Retrieve the app's metadata
    metadata = importlib.metadata.metadata(app_module)

    QtWidgets.QApplication.setApplicationName(metadata["Formal-Name"])

    app = QtWidgets.QApplication(sys.argv)
    main_window = {{ cookiecutter.class_name }}()
    sys.exit(app.exec())
"""

    def pyproject_table_briefcase_app_extra_content(self):
        return """
requires = [
    "PySide6-Essentials~=6.5",
    # "PySide6-Addons~=6.5",
]
test_requires = [
{% if cookiecutter.test_framework == "pytest" %}
    "pytest",
{% endif %}
]
"""

    def pyproject_table_macOS(self):
        return """\
universal_build = true
requires = [
    "std-nslog~=1.0.0",
]
"""

    def pyproject_table_linux(self):
        return """\
requires = [
]
"""

    def pyproject_table_linux_system_debian(self):
        return """\
system_requires = [
]

system_runtime_requires = [
    # Derived from https://doc.qt.io/qt-6/linux-requirements.html
    "libxext6",
    "libxrender1",
    "libx11-xcb1",
    "libxkbcommon-x11-0",
    "libxcb-image0",
    "libxcb-cursor0",
    "libxcb-shape0",
    "libxcb-randr0",
    "libxcb-xfixes0",
    "libxcb-sync1",
    "libxcb-icccm4",
    "libxcb-keysyms1",
    "libfontconfig1",
    "libsm6",
    "libice6",
    "libglib2.0-0",
    "libgl1",
    "libegl1",
    "libdbus-1-3",
]
"""

    def pyproject_table_linux_system_rhel(self):
        return """\
system_requires = [
]

system_runtime_requires = [
    "qt6-qtbase-gui",
]
"""

    def pyproject_table_linux_system_suse(self):
        return """\
system_requires = [
]

system_runtime_requires = [
    "libgthread-2_0-0",
    "libQt6Gui6",
]
"""

    def pyproject_table_linux_system_arch(self):
        return """\
system_requires = [
]

system_runtime_requires = [
    "qt6-base",
]
"""

    def pyproject_table_linux_appimage(self):
        return """\
manylinux = "manylinux_2_28"

system_requires = [
# ?? FIXME
]

linuxdeploy_plugins = [
]
"""

    def pyproject_table_linux_flatpak(self):
        return """\
flatpak_runtime = "org.kde.Platform"
flatpak_runtime_version = "6.6"
flatpak_sdk = "org.kde.Sdk"
"""

    def pyproject_table_windows(self):
        return """\
requires = [
]
"""

    def pyproject_table_iOS(self):
        return """\
supported = false
"""

    def pyproject_table_android(self):
        return """\
supported = false
"""

    def pyproject_table_web(self):
        return """\
supported = false
"""



import sys
from unittest.mock import MagicMock

import pytest

import briefcase.platforms.linux.appimage
from briefcase.console import Console, Log
from briefcase.integrations.docker import Docker, DockerAppContext
from briefcase.integrations.linuxdeploy import LinuxDeploy
from briefcase.integrations.subprocess import Subprocess
from briefcase.platforms.linux.appimage import (
    LinuxAppImageBuildCommand,
    LinuxAppImageCreateCommand,
)


@pytest.fixture
def create_command(tmp_path):
    command = LinuxAppImageCreateCommand(
        logger=Log(),
        console=Console(),
        base_path=tmp_path / "base_path",
        data_path=tmp_path / "briefcase",
    )
    # Mock verified linuxdeploy
    command.tools.linuxdeploy = LinuxDeploy(tools=command.tools)
    return command


def test_binary_path(create_command, first_app_config, tmp_path):
    # Force the architecture to x86_64 for test purposes.
    create_command.tools.host_arch = "x86_64"
    binary_path = create_command.binary_path(first_app_config)

    assert (
        binary_path
        == tmp_path
        / "base_path/build/first-app/linux/appimage/First_App-0.0.1-x86_64.AppImage"
    )


def test_project_path(create_command, first_app_config, tmp_path):
    """The project path is the bundle path."""
    project_path = create_command.project_path(first_app_config)
    bundle_path = create_command.bundle_path(first_app_config)

    expected_path = tmp_path / "base_path/build/first-app/linux/appimage"
    assert expected_path == project_path == bundle_path


def test_distribution_path(create_command, first_app_config, tmp_path):
    # Force the architecture to x86_64 for test purposes.
    create_command.tools.host_arch = "x86_64"
    distribution_path = create_command.distribution_path(first_app_config)

    assert (
        distribution_path == tmp_path / "base_path/dist/First_App-0.0.1-x86_64.AppImage"
    )


@pytest.mark.parametrize(
    "manylinux, tag",
    [
        (None, "appimage"),
        ("manylinux1", "manylinux1-appimage"),
        ("manylinux_2_28", "manylinux_2_28-appimage"),
    ],
)
def test_docker_image_tag(create_command, first_app_config, manylinux, tag):
    if manylinux:
        first_app_config.manylinux = manylinux

    image_tag = create_command.docker_image_tag(first_app_config)

    assert image_tag == f"briefcase/com.example.first-app:{tag}"


def test_docker_image_tag_uppercase_name(
    create_command,
    uppercase_app_config,
    tmp_path,
):
    image_tag = create_command.docker_image_tag(uppercase_app_config)

    assert image_tag == "briefcase/com.example.first-app:appimage"


def test_verify_linux_no_docker(create_command, first_app_config, tmp_path):
    """If Docker is disabled on Linux, the app_context is Subprocess."""
    create_command.tools.host_os = "Linux"
    create_command.use_docker = False

    # Verify the tools
    create_command.verify_tools()
    create_command.verify_app_tools(app=first_app_config)

    # No error and Subprocess is used.
    assert isinstance(create_command.tools[first_app_config].app_context, Subprocess)
    # Docker is not verified.
    assert not hasattr(create_command.tools, "docker")


def test_verify_linux_docker(create_command, first_app_config, tmp_path, monkeypatch):
    """If Docker is enabled on Linux, the Docker alias is set."""
    create_command.tools.host_os = "Linux"
    create_command.use_docker = True
    create_command.extra_docker_build_args = ["--option-one", "--option-two"]

    # Mock Docker tool verification
    mock__version_compat = MagicMock(spec=Docker._version_compat)
    mock__user_access = MagicMock(spec=Docker._user_access)
    mock__buildx_installed = MagicMock(spec=Docker._buildx_installed)
    mock__is_user_mapping_enabled = MagicMock(spec=Docker._is_user_mapping_enabled)
    monkeypatch.setattr(
        briefcase.platforms.linux.appimage.Docker,
        "_version_compat",
        mock__version_compat,
    )
    monkeypatch.setattr(
        briefcase.platforms.linux.appimage.Docker,
        "_user_access",
        mock__user_access,
    )
    monkeypatch.setattr(
        briefcase.platforms.linux.appimage.Docker,
        "_buildx_installed",
        mock__buildx_installed,
    )
    monkeypatch.setattr(
        briefcase.platforms.linux.appimage.Docker,
        "_is_user_mapping_enabled",
        mock__is_user_mapping_enabled,
    )
    mock_docker_app_context_verify = MagicMock(spec=DockerAppContext.verify)
    monkeypatch.setattr(
        briefcase.platforms.linux.appimage.DockerAppContext,
        "verify",
        mock_docker_app_context_verify,
    )

    # Verify the tools
    create_command.verify_tools()
    create_command.verify_app_tools(app=first_app_config)

    # Docker and Docker app context are verified
    mock__version_compat.assert_called_with(tools=create_command.tools)
    mock__user_access.assert_called_with(tools=create_command.tools)
    mock__buildx_installed.assert_called_with(tools=create_command.tools)
    mock__is_user_mapping_enabled.assert_called_with(None)
    assert isinstance(create_command.tools.docker, Docker)
    mock_docker_app_context_verify.assert_called_with(
        tools=create_command.tools,
        app=first_app_config,
        image_tag="briefcase/com.example.first-app:appimage",
        dockerfile_path=tmp_path
        / "base_path/build/first-app/linux/appimage/Dockerfile",
        app_base_path=tmp_path / "base_path",
        host_bundle_path=tmp_path / "base_path/build/first-app/linux/appimage",
        host_data_path=tmp_path / "briefcase",
        python_version=f"3.{sys.version_info.minor}",
        extra_build_args=["--option-one", "--option-two"],
    )


def test_verify_non_linux_docker(
    create_command,
    first_app_config,
    monkeypatch,
    tmp_path,
):
    """If Docker is enabled on non-Linux, the Docker alias is set."""
    create_command.tools.host_os = "Darwin"
    create_command.use_docker = True
    create_command.extra_docker_build_args = ["--option-one", "--option-two"]

    # Mock Docker tool verification
    mock__version_compat = MagicMock(spec=Docker._version_compat)
    mock__user_access = MagicMock(spec=Docker._user_access)
    mock__buildx_installed = MagicMock(spec=Docker._buildx_installed)
    mock__is_user_mapping_enabled = MagicMock(spec=Docker._is_user_mapping_enabled)
    monkeypatch.setattr(
        briefcase.platforms.linux.appimage.Docker,
        "_version_compat",
        mock__version_compat,
    )
    monkeypatch.setattr(
        briefcase.platforms.linux.appimage.Docker,
        "_user_access",
        mock__user_access,
    )
    monkeypatch.setattr(
        briefcase.platforms.linux.appimage.Docker,
        "_buildx_installed",
        mock__buildx_installed,
    )
    monkeypatch.setattr(
        briefcase.platforms.linux.appimage.Docker,
        "_is_user_mapping_enabled",
        mock__is_user_mapping_enabled,
    )
    mock_docker_app_context_verify = MagicMock(spec=DockerAppContext.verify)
    monkeypatch.setattr(
        briefcase.platforms.linux.appimage.DockerAppContext,
        "verify",
        mock_docker_app_context_verify,
    )

    # Verify the tools
    create_command.verify_tools()
    create_command.verify_app_tools(app=first_app_config)

    # Docker and Docker app context are verified
    mock__version_compat.assert_called_with(tools=create_command.tools)
    mock__user_access.assert_called_with(tools=create_command.tools)
    mock__buildx_installed.assert_called_with(tools=create_command.tools)
    mock__is_user_mapping_enabled.assert_called_with(None)
    assert isinstance(create_command.tools.docker, Docker)
    mock_docker_app_context_verify.assert_called_with(
        tools=create_command.tools,
        app=first_app_config,
        image_tag="briefcase/com.example.first-app:appimage",
        dockerfile_path=tmp_path
        / "base_path/build/first-app/linux/appimage/Dockerfile",
        app_base_path=tmp_path / "base_path",
        host_bundle_path=tmp_path / "base_path/build/first-app/linux/appimage",
        host_data_path=tmp_path / "briefcase",
        python_version=f"3.{sys.version_info.minor}",
        extra_build_args=["--option-one", "--option-two"],
    )


def test_clone_options(tmp_path):
    """Docker options are cloned."""
    build_command = LinuxAppImageBuildCommand(
        logger=Log(),
        console=Console(),
        base_path=tmp_path / "base_path",
        data_path=tmp_path / "briefcase",
    )
    build_command.use_docker = True
    build_command.extra_docker_build_args = ["--option-one", "--option-two"]

    create_command = build_command.create_command

    # Confirm the use_docker option has been cloned
    assert create_command.is_clone
    assert create_command.use_docker
    assert create_command.extra_docker_build_args == ["--option-one", "--option-two"]



import subprocess

import pytest

from briefcase.console import LogLevel
from briefcase.exceptions import BriefcaseCommandError


@pytest.mark.parametrize("tool_debug_mode", (True, False))
def test_bundle(flatpak, tool_debug_mode, tmp_path):
    """A Flatpak project can be bundled."""
    # Enable verbose tool logging
    if tool_debug_mode:
        flatpak.tools.logger.verbosity = LogLevel.DEEP_DEBUG

    flatpak.bundle(
        repo_url="https://example.com/flatpak",
        bundle_identifier="com.example.my-app",
        app_name="my-app",
        version="1.2.3",
        build_path=tmp_path / "build",
        output_path=tmp_path / "output/MyApp.flatpak",
    )

    # The expected call was made
    flatpak.tools.subprocess.run.assert_called_once_with(
        [
            "flatpak",
            "build-bundle",
            "--runtime-repo",
            "https://example.com/flatpak",
            "repo",
            tmp_path / "output/MyApp.flatpak",
            "com.example.my-app",
            "1.2.3",
        ]
        + (["--verbose"] if tool_debug_mode else []),
        check=True,
        cwd=tmp_path / "build",
    )


def test_bundle_fail(flatpak, tmp_path):
    """If bundling fails, an error is raised."""
    flatpak.tools.subprocess.run.side_effect = subprocess.CalledProcessError(
        cmd="flatpak build-bundle", returncode=1
    )

    with pytest.raises(
        BriefcaseCommandError,
        match=r"Unable to build a Flatpak bundle for app my-app.",
    ):
        flatpak.bundle(
            repo_url="https://example.com/flatpak",
            bundle_identifier="com.example.my-app",
            app_name="my-app",
            version="1.2.3",
            build_path=tmp_path / "build",
            output_path=tmp_path / "output/MyApp.flatpak",
        )



import os
import platform
import shutil
import sys
from unittest.mock import MagicMock

import pytest

from briefcase.console import LogLevel
from briefcase.exceptions import (
    BriefcaseCommandError,
    IncompatibleToolError,
    MissingToolError,
    NetworkFailure,
    UnsupportedHostError,
)
from briefcase.integrations.android_sdk import AndroidSDK
from briefcase.integrations.base import ToolCache

from ..conftest import SDK_MGR_DL_VER, SDK_MGR_VER

SDKMANAGER_FILENAME = (
    "sdkmanager.bat" if platform.system() == "Windows" else "sdkmanager"
)


@pytest.fixture
def mock_tools(mock_tools) -> ToolCache:
    # Mock the os environment, but copy over other key functions.
    mock_tools.os.environ = {}
    mock_tools.os.fsdecode = os.fsdecode
    mock_tools.os.access = os.access
    mock_tools.os.X_OK = os.X_OK
    mock_tools.os.unlink = os.unlink

    # Identify the host platform
    mock_tools._test_download_tag = {
        "Windows": "win",
        "Darwin": "mac",
        "Linux": "linux",
    }[mock_tools.host_os]

    # Use the original module rmtree implementation
    mock_tools.shutil.rmtree = shutil.rmtree

    return mock_tools


def mock_unpack(filename, extract_dir, filter=None):
    # Create a file that would have been created by unpacking the archive
    # This includes the duplicated "cmdline-tools" folder name
    (extract_dir / "cmdline-tools/bin").mkdir(parents=True)
    (extract_dir / "cmdline-tools/bin/sdkmanager").touch(mode=0o644)
    (extract_dir / "cmdline-tools/bin/avdmanager").touch(mode=0o644)
    # Include an extra tool that is already executable.
    (extract_dir / "cmdline-tools/bin/other").touch(mode=0o755)


def accept_license(android_sdk_root_path):
    """Generate a side effect method that will accept a license."""

    def _side_effect(*args, **kwargs):
        license_dir = android_sdk_root_path / "licenses"
        license_dir.mkdir(parents=True)
        (license_dir / "android-sdk-license").touch()

    return _side_effect


def test_short_circuit(mock_tools):
    """Tool is not created if already cached."""
    mock_tools.android_sdk = "tool"

    tool = AndroidSDK.verify(mock_tools)

    assert tool == "tool"
    assert tool == mock_tools.android_sdk


def test_unsupported_os(mock_tools):
    """When host OS is not supported, an error is raised."""
    mock_tools.host_os = "wonky"

    with pytest.raises(
        UnsupportedHostError,
        match=f"{AndroidSDK.name} is not supported on wonky",
    ):
        AndroidSDK.verify(mock_tools)


def test_unsupported_arch(mock_tools):
    """When the architecture is not supported, an error is raised."""
    mock_tools.host_arch = "IA-64"

    with pytest.raises(
        IncompatibleToolError,
        match="Briefcase cannot install Android SDK on this machine.",
    ):
        AndroidSDK.verify(mock_tools)


@pytest.mark.parametrize(
    "host_os, host_arch",
    [
        ("Darwin", "arm64"),
        ("Darwin", "x86_64"),
        ("Linux", "x86_64"),
        ("Linux", "aarch64"),
        ("Windows", "AMD64"),
    ],
)
def test_supported_os_arch(mock_tools, host_os, host_arch, tmp_path):
    """The supported OS/arch combinations do not error."""
    mock_tools.host_os = host_os
    mock_tools.host_arch = host_arch

    # Create `sdkmanager` and the license file.
    android_sdk_root_path = tmp_path / "tools/android_sdk"
    tools_bin = android_sdk_root_path / f"cmdline-tools/{SDK_MGR_VER}/bin"
    tools_bin.mkdir(parents=True, mode=0o755)
    if host_os == "Windows":
        sdk_manager = tools_bin / "sdkmanager.bat"
        sdk_manager.touch()
    else:
        sdk_manager = tools_bin / "sdkmanager"
        sdk_manager.touch(mode=0o755)

    # Pre-accept the license
    accept_license(android_sdk_root_path)()

    assert isinstance(AndroidSDK.verify(mock_tools), AndroidSDK)


def test_succeeds_immediately_in_happy_path(mock_tools, tmp_path):
    """If verify is invoked on a path containing an Android SDK, it does nothing."""
    # If `sdkmanager` exists and has the right permissions, and `android-sdk-license`
    # exists, verify() should succeed, create no subprocesses, make no requests, and
    # return an SDK wrapper.

    # Create `sdkmanager` and the license file.
    android_sdk_root_path = tmp_path / "tools/android_sdk"
    tools_bin = android_sdk_root_path / "cmdline-tools" / SDK_MGR_VER / "bin"
    tools_bin.mkdir(parents=True, mode=0o755)
    (tools_bin / SDKMANAGER_FILENAME).touch(mode=0o755)

    # Pre-accept the license
    accept_license(android_sdk_root_path)()

    # Expect verify() to succeed
    sdk = AndroidSDK.verify(mock_tools)

    # No calls to download, run or unpack anything.
    mock_tools.file.download.assert_not_called()
    mock_tools.subprocess.run.assert_not_called()
    mock_tools.shutil.unpack_archive.assert_not_called()

    # The returned SDK has the expected root path.
    assert sdk.root_path == android_sdk_root_path


@pytest.mark.parametrize("env_var", ["ANDROID_HOME", "ANDROID_SDK_ROOT"])
def test_user_provided_sdk(mock_tools, env_var, tmp_path, capsys):
    """If the user environment specifies a valid Android SDK, it is used."""
    # Increase the log level.
    mock_tools.logger.verbosity = LogLevel.DEBUG

    # Create `sdkmanager` and the license file.
    existing_android_sdk_root_path = tmp_path / "other_sdk"
    tools_bin = existing_android_sdk_root_path / "cmdline-tools" / SDK_MGR_VER / "bin"
    tools_bin.mkdir(parents=True, mode=0o755)
    (tools_bin / SDKMANAGER_FILENAME).touch(mode=0o755)

    # Pre-accept the license
    accept_license(existing_android_sdk_root_path)()

    # Set the environment to specify ANDROID_SDK_ROOT
    mock_tools.os.environ = {env_var: os.fsdecode(existing_android_sdk_root_path)}

    # Expect verify() to succeed
    sdk = AndroidSDK.verify(mock_tools)

    # No calls to download or unpack anything.
    mock_tools.file.download.assert_not_called()
    mock_tools.shutil.unpack_archive.assert_not_called()

    # The returned SDK has the expected root path.
    assert sdk.root_path == existing_android_sdk_root_path

    # User is warned about using ANDROID_SDK_ROOT
    if env_var == "ANDROID_SDK_ROOT":
        assert "Using Android SDK from ANDROID_SDK_ROOT" in capsys.readouterr().out
    else:
        assert capsys.readouterr().out == (
            "\n[Android SDK] Evaluating ANDROID_HOME...\n"
            f"ANDROID_HOME={tmp_path / 'other_sdk'}\n"
            f"Using Android SDK at {tmp_path / 'other_sdk'}\n"
        )


def test_consistent_user_provided_sdk(mock_tools, tmp_path, capsys):
    """If the user environment specifies a valid Android SDK in both ANDROID_HOME and
    ANDROID_SDK_ROOT, it is used."""
    # Increase the log level.
    mock_tools.logger.verbosity = LogLevel.DEBUG

    # Create `sdkmanager` and the license file.
    existing_android_sdk_root_path = tmp_path / "other_sdk"
    tools_bin = existing_android_sdk_root_path / "cmdline-tools" / SDK_MGR_VER / "bin"
    tools_bin.mkdir(parents=True, mode=0o755)
    (tools_bin / SDKMANAGER_FILENAME).touch(mode=0o755)

    # Pre-accept the license
    accept_license(existing_android_sdk_root_path)()

    # Set the environment to specify ANDROID_SDK_ROOT
    mock_tools.os.environ = {
        "ANDROID_HOME": os.fsdecode(existing_android_sdk_root_path),
        "ANDROID_SDK_ROOT": os.fsdecode(existing_android_sdk_root_path),
    }

    # Expect verify() to succeed
    sdk = AndroidSDK.verify(mock_tools)

    # No calls to download or unpack anything.
    mock_tools.file.download.assert_not_called()
    mock_tools.shutil.unpack_archive.assert_not_called()

    # The returned SDK has the expected root path.
    assert sdk.root_path == existing_android_sdk_root_path

    # User is not warned about configuration
    assert capsys.readouterr().out == (
        "\n[Android SDK] Evaluating ANDROID_HOME...\n"
        f"ANDROID_HOME={tmp_path / 'other_sdk'}\n"
        f"Using Android SDK at {tmp_path / 'other_sdk'}\n"
    )


def test_inconsistent_user_provided_sdk(mock_tools, tmp_path, capsys):
    """A warning is logged if ANDROID_HOME and ANDROID_SDK_ROOT are not the same."""
    # Increase the log level.
    mock_tools.logger.verbosity = LogLevel.DEBUG

    # Create `sdkmanager` and the license file.
    existing_android_sdk_root_path = tmp_path / "other_sdk"
    tools_bin = existing_android_sdk_root_path / "cmdline-tools" / SDK_MGR_VER / "bin"
    tools_bin.mkdir(parents=True, mode=0o755)
    (tools_bin / SDKMANAGER_FILENAME).touch(mode=0o755)

    # Pre-accept the license
    accept_license(existing_android_sdk_root_path)()

    # Set the environment to specify ANDROID_HOME and ANDROID_SDK_ROOT
    mock_tools.os.environ = {
        "ANDROID_HOME": os.fsdecode(existing_android_sdk_root_path),
        "ANDROID_SDK_ROOT": "/opt/sdk2",
    }

    # Expect verify() to succeed
    sdk = AndroidSDK.verify(mock_tools)

    # No calls to download or unpack anything.
    mock_tools.file.download.assert_not_called()
    mock_tools.shutil.unpack_archive.assert_not_called()

    # The returned SDK has the expected root path.
    assert sdk.root_path == existing_android_sdk_root_path

    # User is warned about using ANDROID_SDK_ROOT
    assert (
        "ANDROID_HOME and ANDROID_SDK_ROOT are inconsistent" in capsys.readouterr().out
    )


@pytest.mark.parametrize("env_var", ["ANDROID_HOME", "ANDROID_SDK_ROOT"])
def test_invalid_user_provided_sdk(mock_tools, env_var, tmp_path, capsys):
    """If the user's environment specifies an invalid Android SDK, it is ignored."""
    # Create `sdkmanager` and the license file
    # for the *briefcase* managed version of the SDK.
    android_sdk_root_path = tmp_path / "tools/android_sdk"
    tools_bin = android_sdk_root_path / "cmdline-tools" / SDK_MGR_VER / "bin"
    tools_bin.mkdir(parents=True, mode=0o755)
    (tools_bin / SDKMANAGER_FILENAME).touch(mode=0o755)

    # Pre-accept the license
    accept_license(android_sdk_root_path)()

    # Set the environment to specify an ANDROID_SDK_ROOT that doesn't exist
    mock_tools.os.environ = {env_var: os.fsdecode(tmp_path / "other_sdk")}

    # Expect verify() to succeed
    sdk = AndroidSDK.verify(mock_tools)

    # No calls to download, run or unpack anything.
    mock_tools.file.download.assert_not_called()
    mock_tools.subprocess.run.assert_not_called()
    mock_tools.shutil.unpack_archive.assert_not_called()

    # The returned SDK has the expected root path.
    assert sdk.root_path == android_sdk_root_path

    # User is informed about invalid env var setting
    assert f"{env_var} does not point to an Android SDK" in capsys.readouterr().out


@pytest.mark.parametrize("env_var", ["ANDROID_HOME", "ANDROID_SDK_ROOT"])
def test_user_provided_sdk_wrong_cmdline_tools_ver(
    mock_tools,
    env_var,
    tmp_path,
    capsys,
):
    """If the user's environment specifies an Android SDK without the expected cmdline
    tools and the cmdline-tools install fails, the Briefcase SDK is used."""
    # Create `sdkmanager` and the license file for the *user's* version of the SDK
    user_sdk_path = tmp_path / "other_sdk"
    user_tools_bin = user_sdk_path / "cmdline-tools/6.0/bin"
    user_tools_bin.mkdir(parents=True, mode=0o755)
    (user_tools_bin / SDKMANAGER_FILENAME).touch(mode=0o755)

    # Create `sdkmanager` and the license file
    # for the *briefcase* managed version of the SDK.
    android_sdk_root_path = tmp_path / "tools/android_sdk"
    tools_bin = android_sdk_root_path / "cmdline-tools" / SDK_MGR_VER / "bin"
    tools_bin.mkdir(parents=True, mode=0o755)
    (tools_bin / SDKMANAGER_FILENAME).touch(mode=0o755)

    # Pre-accept the license
    accept_license(android_sdk_root_path)()

    # Set the environment to specify an ANDROID_SDK_ROOT that doesn't exist
    mock_tools.os.environ = {env_var: os.fsdecode(user_sdk_path)}

    # Mock `cmdline-tools/latest` directory not existing
    mock_tools.subprocess.run.side_effect = OSError

    # Expect verify() to succeed
    sdk = AndroidSDK.verify(mock_tools)

    # No calls to download and nothing unpacked
    mock_tools.file.download.assert_not_called()
    mock_tools.shutil.unpack_archive.assert_not_called()

    # Required Command-line Tools installed
    mock_tools.subprocess.run.assert_called_once_with(
        [
            tmp_path
            / "other_sdk"
            / "cmdline-tools"
            / "latest"
            / "bin"
            / SDKMANAGER_FILENAME,
            f"cmdline-tools;{SDK_MGR_VER}",
        ],
        check=True,
        stream_output=False,
    )

    # The returned SDK has the expected root path.
    assert sdk.root_path == android_sdk_root_path

    # User is informed about failed install and invalid SDK
    output = capsys.readouterr().out
    assert f"Failed to install cmdline-tools;{SDK_MGR_VER}" in output
    assert "Incompatible Command-Line Tools Version" in output


@pytest.mark.parametrize("env_var", ["ANDROID_HOME", "ANDROID_SDK_ROOT"])
def test_user_provided_sdk_with_latest_cmdline_tools(
    mock_tools,
    env_var,
    tmp_path,
    capsys,
):
    """If the user's environment specifies an Android SDK without the expected cmdline
    tools, the required cmdline-tools is installed in to it."""
    # Create `sdkmanager` and the license file for the *user's* version of the SDK
    user_sdk_path = tmp_path / "other_sdk"
    user_tools_bin = user_sdk_path / "cmdline-tools/latest/bin"
    user_tools_bin.mkdir(parents=True, mode=0o755)
    (user_tools_bin / SDKMANAGER_FILENAME).touch(mode=0o755)

    # Pre-accept the license
    accept_license(user_sdk_path)()

    # Set the environment to specify an ANDROID_SDK_ROOT that doesn't exist
    mock_tools.os.environ = {env_var: os.fsdecode(user_sdk_path)}

    # Expect verify() to succeed
    sdk = AndroidSDK.verify(mock_tools)

    # No calls to download and nothing unpacked
    mock_tools.file.download.assert_not_called()
    mock_tools.shutil.unpack_archive.assert_not_called()

    # Required Command-line Tools installed
    mock_tools.subprocess.run.assert_called_once_with(
        [
            tmp_path
            / "other_sdk"
            / "cmdline-tools"
            / "latest"
            / "bin"
            / SDKMANAGER_FILENAME,
            f"cmdline-tools;{SDK_MGR_VER}",
        ],
        check=True,
        stream_output=False,
    )

    # The returned SDK has the expected root path.
    assert sdk.root_path == user_sdk_path


def test_consistent_invalid_user_provided_sdk(mock_tools, tmp_path, capsys):
    """If the user's environment specifies an invalid Android SDK in both ANDROID_HOME
    and ANDROID_SDK_ROOT, they are ignored."""

    # Create `sdkmanager` and the license file
    # for the *briefcase* managed version of the SDK.
    android_sdk_root_path = tmp_path / "tools/android_sdk"
    tools_bin = android_sdk_root_path / "cmdline-tools" / SDK_MGR_VER / "bin"
    tools_bin.mkdir(parents=True, mode=0o755)
    (tools_bin / SDKMANAGER_FILENAME).touch(mode=0o755)

    # Pre-accept the license
    accept_license(android_sdk_root_path)()

    # Set the environment to specify an ANDROID_SDK_ROOT that doesn't exist
    mock_tools.os.environ = {
        "ANDROID_HOME": os.fsdecode(tmp_path / "other_sdk"),
        "ANDROID_SDK_ROOT": os.fsdecode(tmp_path / "other_sdk"),
    }

    # Expect verify() to succeed
    sdk = AndroidSDK.verify(mock_tools)

    # No calls to download, run or unpack anything.
    mock_tools.file.download.assert_not_called()
    mock_tools.subprocess.run.assert_not_called()
    mock_tools.shutil.unpack_archive.assert_not_called()

    # The returned SDK has the expected root path.
    assert sdk.root_path == android_sdk_root_path

    # User is informed about invalid env var setting
    assert "ANDROID_HOME does not point to an Android SDK" in capsys.readouterr().out


def test_inconsistent_invalid_user_provided_sdk(mock_tools, tmp_path, capsys):
    """If the user's environment specifies an invalid Android SDK in both ANDROID_HOME
    and ANDROID_SDK_ROOT...and they are both different, they are ignored."""

    # Create `sdkmanager` and the license file
    # for the *briefcase* managed version of the SDK.
    android_sdk_root_path = tmp_path / "tools/android_sdk"
    tools_bin = android_sdk_root_path / "cmdline-tools" / SDK_MGR_VER / "bin"
    tools_bin.mkdir(parents=True, mode=0o755)
    (tools_bin / SDKMANAGER_FILENAME).touch(mode=0o755)

    # Pre-accept the license
    accept_license(android_sdk_root_path)()

    # Set the environment to specify an ANDROID_SDK_ROOT that doesn't exist
    mock_tools.os.environ = {
        "ANDROID_HOME": os.fsdecode(tmp_path / "other_sdk1"),
        "ANDROID_SDK_ROOT": os.fsdecode(tmp_path / "other_sdk2"),
    }

    # Expect verify() to succeed
    sdk = AndroidSDK.verify(mock_tools)

    # No calls to download, run or unpack anything.
    mock_tools.file.download.assert_not_called()
    mock_tools.subprocess.run.assert_not_called()
    mock_tools.shutil.unpack_archive.assert_not_called()

    # The returned SDK has the expected root path.
    assert sdk.root_path == android_sdk_root_path

    # User is informed about invalid env var setting
    output = capsys.readouterr().out
    assert "ANDROID_HOME and ANDROID_SDK_ROOT are inconsistent" in output
    assert "ANDROID_HOME does not point to an Android SDK" in output


def test_download_sdk(mock_tools, tmp_path, capsys):
    """If an SDK is not available, one will be downloaded."""
    android_sdk_root_path = tmp_path / "tools/android_sdk"
    cmdline_tools_base_path = android_sdk_root_path / "cmdline-tools"

    # The download will produce a cached file.
    cache_file = MagicMock()
    mock_tools.file.download = MagicMock(return_value=cache_file)

    # Calling unpack will create files
    mock_tools.shutil.unpack_archive.side_effect = mock_unpack

    # Set up a side effect for accepting the license
    mock_tools.subprocess.run.side_effect = accept_license(android_sdk_root_path)

    # Call `verify()`
    sdk = AndroidSDK.verify(mock_tools)

    # Validate that the SDK was downloaded and unpacked
    url = (
        "https://dl.google.com/android/repository/"
        f"commandlinetools-{mock_tools._test_download_tag}-{SDK_MGR_DL_VER}_latest.zip"
    )
    mock_tools.file.download.assert_called_once_with(
        url=url,
        download_path=mock_tools.base_path,
        role="Android SDK Command-Line Tools",
    )

    mock_tools.shutil.unpack_archive.assert_called_once_with(
        filename=cache_file,
        extract_dir=cmdline_tools_base_path,
        **({"filter": "data"} if sys.version_info >= (3, 12) else {}),
    )

    # The cached file will be deleted
    cache_file.unlink.assert_called_once_with()

    # The commandline tools path exists
    assert sdk.cmdline_tools_path.is_dir()

    if platform.system() != "Windows":
        # On non-Windows, ensure the unpacked binary was made executable
        assert os.access(
            cmdline_tools_base_path / SDK_MGR_VER / "bin/sdkmanager",
            os.X_OK,
        )

    # The license has been accepted
    assert (android_sdk_root_path / "licenses/android-sdk-license").exists()

    # The returned SDK has the expected root path.
    assert sdk.root_path == android_sdk_root_path

    assert "The Android SDK was not found" in capsys.readouterr().out


def test_upgrade_existing_sdk(mock_tools, tmp_path, capsys):
    """An existing SDK is upgraded if the required version of cmdline-tools isn't
    installed."""
    android_sdk_root_path = tmp_path / "tools/android_sdk"
    cmdline_tools_base_path = android_sdk_root_path / "cmdline-tools"

    # Mock an existing cmdline-tools install
    (cmdline_tools_base_path / "latest").mkdir(parents=True)
    (cmdline_tools_base_path / "8092744").touch()

    # The download will produce a cached file.
    cache_file = MagicMock()
    mock_tools.file.download = MagicMock(return_value=cache_file)

    # Calling unpack will create files
    mock_tools.shutil.unpack_archive.side_effect = mock_unpack

    # Set up a side effect for accepting the license
    mock_tools.subprocess.run.side_effect = accept_license(android_sdk_root_path)

    # Call `verify()`
    sdk = AndroidSDK.verify(mock_tools)

    # Validate that the SDK was downloaded and unpacked
    url = (
        "https://dl.google.com/android/repository/"
        f"commandlinetools-{mock_tools._test_download_tag}-{SDK_MGR_DL_VER}_latest.zip"
    )
    mock_tools.file.download.assert_called_once_with(
        url=url,
        download_path=mock_tools.base_path,
        role="Android SDK Command-Line Tools",
    )

    mock_tools.shutil.unpack_archive.assert_called_once_with(
        filename=cache_file,
        extract_dir=cmdline_tools_base_path,
        **({"filter": "data"} if sys.version_info >= (3, 12) else {}),
    )

    # The cached file will be deleted
    cache_file.unlink.assert_called_once_with()

    # The commandline tools path exists
    assert sdk.cmdline_tools_path.is_dir()

    if platform.system() != "Windows":
        # On non-Windows, ensure the unpacked binary was made executable
        assert os.access(
            cmdline_tools_base_path / SDK_MGR_VER / "bin/sdkmanager",
            os.X_OK,
        )

    # The license has been accepted
    assert (android_sdk_root_path / "licenses/android-sdk-license").exists()

    # The returned SDK has the expected root path.
    assert sdk.root_path == android_sdk_root_path

    # Ensure old cmdline-tools installs are cleaned up
    assert not (cmdline_tools_base_path / "latest").exists()
    assert not (cmdline_tools_base_path / "8092744").exists()

    assert "Upgrading Android SDK..." in capsys.readouterr().out


def test_download_sdk_legacy_install(mock_tools, tmp_path):
    """If the legacy SDK tools are present, they will be deleted."""
    android_sdk_root_path = tmp_path / "tools/android_sdk"
    cmdline_tools_base_path = android_sdk_root_path / "cmdline-tools"

    # Create files that mock the existence of the *old* SDK tools.
    sdk_tools_base_path = android_sdk_root_path / "tools"
    (sdk_tools_base_path / "bin").mkdir(parents=True)
    (sdk_tools_base_path / "bin/sdkmanager").touch(mode=0o755)
    (sdk_tools_base_path / "bin/avdmanager").touch(mode=0o755)

    # Create some of the tools that have locations that overlap
    # between legacy and new.
    emulator_path = android_sdk_root_path / "emulator"
    emulator_path.mkdir(parents=True)
    (emulator_path / "emulator").touch(mode=0o755)

    # The download will produce a cached file.
    cache_file = MagicMock()
    mock_tools.file.download = MagicMock(return_value=cache_file)

    # Calling unpack will create files
    mock_tools.shutil.unpack_archive.side_effect = mock_unpack

    # Set up a side effect for accepting the license
    mock_tools.subprocess.run.side_effect = accept_license(android_sdk_root_path)

    # Call `verify()`
    sdk = AndroidSDK.verify(mock_tools)

    # Validate that the SDK was downloaded and unpacked
    url = (
        "https://dl.google.com/android/repository/"
        f"commandlinetools-{mock_tools._test_download_tag}-{SDK_MGR_DL_VER}_latest.zip"
    )
    mock_tools.file.download.assert_called_once_with(
        url=url,
        download_path=mock_tools.base_path,
        role="Android SDK Command-Line Tools",
    )

    mock_tools.shutil.unpack_archive.assert_called_once_with(
        filename=cache_file,
        extract_dir=cmdline_tools_base_path,
        **({"filter": "data"} if sys.version_info >= (3, 12) else {}),
    )

    # The cached file will be deleted
    cache_file.unlink.assert_called_once_with()

    # The commandline tools path exists, in both "latest" and versioned form
    assert sdk.cmdline_tools_path.is_dir()

    if platform.system() != "Windows":
        # On non-Windows, ensure the unpacked binary was made executable
        assert os.access(
            cmdline_tools_base_path / SDK_MGR_VER / "bin/sdkmanager", os.X_OK
        )

    # The legacy SDK tools have been removed
    assert not sdk_tools_base_path.exists()
    assert not emulator_path.exists()

    # The license has been accepted
    assert (android_sdk_root_path / "licenses/android-sdk-license").exists()

    # The returned SDK has the expected root path.
    assert sdk.root_path == android_sdk_root_path


def test_no_install(mock_tools, tmp_path):
    """If an SDK is not available, and install is not requested, an error is raised."""

    # Call `verify()`
    with pytest.raises(MissingToolError):
        AndroidSDK.verify(mock_tools, install=False)

    assert mock_tools.file.download.call_count == 0


def test_download_sdk_if_sdkmanager_not_executable(mock_tools, tmp_path):
    """An SDK will be downloaded and unpacked if `tools/bin/sdkmanager` exists but does
    not have its permissions set properly."""
    android_sdk_root_path = tmp_path / "tools/android_sdk"
    cmdline_tools_base_path = android_sdk_root_path / "cmdline-tools"

    # Create pre-existing non-executable `sdkmanager`.
    (cmdline_tools_base_path / SDK_MGR_VER / "bin").mkdir(parents=True)
    (cmdline_tools_base_path / SDK_MGR_VER / "bin/sdkmanager").touch(mode=0o644)
    (cmdline_tools_base_path / SDK_MGR_DL_VER).touch()

    # The download will produce a cached file
    cache_file = MagicMock()
    mock_tools.file.download = MagicMock(return_value=cache_file)

    # Calling unpack will create files
    mock_tools.shutil.unpack_archive.side_effect = mock_unpack

    # Set up a side effect for accepting the license
    mock_tools.subprocess.run.side_effect = accept_license(android_sdk_root_path)

    # Call `verify()`
    sdk = AndroidSDK.verify(mock_tools)

    # Validate that the SDK was downloaded and unpacked
    url = (
        "https://dl.google.com/android/repository/"
        f"commandlinetools-{mock_tools._test_download_tag}-{SDK_MGR_DL_VER}_latest.zip"
    )
    mock_tools.file.download.assert_called_once_with(
        url=url,
        download_path=mock_tools.base_path,
        role="Android SDK Command-Line Tools",
    )

    mock_tools.shutil.unpack_archive.assert_called_once_with(
        filename=cache_file,
        extract_dir=cmdline_tools_base_path,
        **({"filter": "data"} if sys.version_info >= (3, 12) else {}),
    )

    # The cached file will be deleted
    cache_file.unlink.assert_called_once_with()

    # The license has been accepted
    assert (android_sdk_root_path / "licenses/android-sdk-license").exists()

    # The returned SDK has the expected root path.
    assert sdk.root_path == android_sdk_root_path


def test_raises_networkfailure_on_connectionerror(mock_tools):
    """If an error occurs downloading the ZIP file, and error is raised."""
    mock_tools.file.download = MagicMock(side_effect=NetworkFailure("mock"))

    with pytest.raises(NetworkFailure, match="Unable to mock"):
        AndroidSDK.verify(mock_tools)

    # The download was attempted
    url = (
        "https://dl.google.com/android/repository/"
        f"commandlinetools-{mock_tools._test_download_tag}-{SDK_MGR_DL_VER}_latest.zip"
    )
    mock_tools.file.download.assert_called_once_with(
        url=url,
        download_path=mock_tools.base_path,
        role="Android SDK Command-Line Tools",
    )
    # But no unpack occurred
    assert mock_tools.shutil.unpack_archive.call_count == 0


def test_detects_bad_zipfile(mock_tools, tmp_path):
    """If the ZIP file is corrupted, an error is raised."""
    android_sdk_root_path = tmp_path / "tools/android_sdk"

    cache_file = MagicMock()
    mock_tools.file.download = MagicMock(return_value=cache_file)

    # But the unpack will fail.
    mock_tools.shutil.unpack_archive.side_effect = shutil.ReadError

    with pytest.raises(BriefcaseCommandError):
        AndroidSDK.verify(mock_tools)

    # The download attempt was made.
    url = (
        "https://dl.google.com/android/repository/"
        f"commandlinetools-{mock_tools._test_download_tag}-{SDK_MGR_DL_VER}_latest.zip"
    )
    mock_tools.file.download.assert_called_once_with(
        url=url,
        download_path=mock_tools.base_path,
        role="Android SDK Command-Line Tools",
    )
    mock_tools.shutil.unpack_archive.assert_called_once_with(
        filename=cache_file,
        extract_dir=android_sdk_root_path / "cmdline-tools",
        **({"filter": "data"} if sys.version_info >= (3, 12) else {}),
    )



from __future__ import annotations

import os
import shutil
import subprocess
from pathlib import Path

from briefcase.exceptions import (
    BriefcaseCommandError,
    IncompatibleToolError,
    MissingToolError,
)
from briefcase.integrations.base import ManagedTool, ToolCache


class JDK(ManagedTool):
    name = "java"
    full_name = "Java JDK"

    # Latest OpenJDK as of April 2024: https://adoptium.net/temurin/releases/
    JDK_MAJOR_VER = "17"
    JDK_RELEASE = "17.0.11"
    JDK_BUILD = "9"
    JDK_INSTALL_DIR_NAME = f"java{JDK_MAJOR_VER}"

    def __init__(self, tools: ToolCache, java_home: Path):
        super().__init__(tools=tools)
        self.java_home = java_home

    @property
    def OpenJDK_download_url(self):
        """The OpenJDK download URL appropriate for the current machine."""
        system_arch = self.tools.host_arch
        # use a 32bit JDK if using 32bit Python on 64bit hardware
        if self.tools.is_32bit_python and self.tools.host_arch == "aarch64":
            system_arch = "armv7l"

        try:
            platform, arch, extension = {
                "Darwin": {
                    "arm64": ("mac", "aarch64", "tar.gz"),
                    "x86_64": ("mac", "x64", "tar.gz"),
                },
                "Linux": {
                    "armv7l": ("linux", "arm", "tar.gz"),
                    "armv8l": ("linux", "arm", "tar.gz"),
                    "aarch64": ("linux", "aarch64", "tar.gz"),
                    "x86_64": ("linux", "x64", "tar.gz"),
                },
                "Windows": {
                    "AMD64": ("windows", "x64", "zip"),
                },
            }[self.tools.host_os][system_arch]
        except KeyError as e:
            raise IncompatibleToolError(tool=self.full_name, env_var="JAVA_HOME") from e

        return (
            f"https://github.com/adoptium/temurin{self.JDK_MAJOR_VER}-binaries/"
            f"releases/download/jdk-{self.JDK_RELEASE}+{self.JDK_BUILD}/"
            f"OpenJDK{self.JDK_MAJOR_VER}U-jdk_{arch}_{platform}_hotspot_"
            f"{self.JDK_RELEASE}_{self.JDK_BUILD}.{extension}"
        )

    @classmethod
    def version_from_path(cls, tools: ToolCache, java_path: str | Path) -> str:
        """Return a JDK's version from a path by running ``<java_path>/bin/javac``.

        This will fail if the path contains a JRE instead of a JDK.

        Several exceptions can be raised for issues:
         - OSError - the ``javac`` executable doesn't exist
         - CalledProcessError - ``javac`` returned a non-zero value
         - IndexError - unparsable version value from ``javac``

        :param tools: ToolCache of available tools
        :param java_path: File path to a candidate JDK install
        :return: JDK release version; e.g. "17.0.X"
        """
        output = tools.subprocess.check_output(
            [Path(java_path) / "bin/javac", "-version"]
        )
        # javac's output should look like "javac 17.0.X\n"
        return output.strip("\n").split(" ")[1]

    @classmethod
    def verify_install(cls, tools: ToolCache, install: bool = True, **kwargs) -> JDK:
        """Verify that a Java JDK exists.

        If ``JAVA_HOME`` is set, try that version. If it is a JRE, or its *not*
        a Java JDK, download one.

        On macOS, also try invoking /usr/libexec/java_home. If that location
        points to a Java JDK, use it.

        Otherwise, download a JDK from OpenJDK and unpack it into the
        briefcase data directory.

        :param tools: ToolCache of available tools
        :param install: Should the tool be installed if it is not found?
        :returns: A valid Java JDK wrapper. If a JDK is not available, and was
            not installed, raises MissingToolError.
        """
        # short circuit since already verified and available
        if hasattr(tools, "java"):
            return tools.java

        java = None
        install_message = None

        if java_home := tools.os.environ.get("JAVA_HOME", ""):
            tools.logger.debug("Evaluating JAVA_HOME...", prefix=cls.full_name)
            tools.logger.debug(f"JAVA_HOME={java_home}")
            try:
                version_str = cls.version_from_path(tools, java_home)
                if version_str.split(".")[0] == cls.JDK_MAJOR_VER:
                    java = JDK(tools, java_home=Path(java_home))
                else:
                    install_message = f"""
*************************************************************************
** WARNING: JAVA_HOME does not point to a Java {cls.JDK_MAJOR_VER} JDK                  **
*************************************************************************

    Android requires a Java {cls.JDK_MAJOR_VER} JDK, but the location pointed to by the
    JAVA_HOME environment variable:

    {java_home}

    isn't a Java {cls.JDK_MAJOR_VER} JDK (it appears to be Java {version_str}).

    Briefcase will proceed using its own JDK instance.

*************************************************************************
"""

            except OSError:
                install_message = f"""
*************************************************************************
** WARNING: JAVA_HOME does not point to a JDK                          **
*************************************************************************

    The location pointed to by the JAVA_HOME environment variable:

    {java_home}

    does not appear to be a JDK. It may be a Java Runtime Environment.

    If JAVA_HOME is a JDK, ensure it is the root directory of the JDK
    instance such that $JAVA_HOME/bin/javac is a valid filepath.

    Briefcase will proceed using its own JDK instance.

*************************************************************************
"""

            except subprocess.CalledProcessError:
                install_message = f"""
*************************************************************************
** WARNING: Unable to invoke the Java compiler                         **
*************************************************************************

    Briefcase received an unexpected error when trying to invoke javac,
    the Java compiler, at the location indicated by the JAVA_HOME
    environment variable.

    Briefcase will continue by downloading and using its own JDK.

    Please report this as a bug at:

        https://github.com/beeware/briefcase/issues/new


    In your report, please including the output from running:

        {java_home}/bin/javac -version

    from the command prompt.

*************************************************************************
"""

            except IndexError:
                install_message = f"""
*************************************************************************
** WARNING: Unable to determine the version of Java that is installed  **
*************************************************************************

    Briefcase was unable to interpret the version information returned
    by the Java compiler at the location indicated by the JAVA_HOME
    environment variable.

    Briefcase will continue by downloading and using its own JDK.

    Please report this as a bug at:

        https://github.com/beeware/briefcase/issues/new


    In your report, please including the output from running:

        {java_home}/bin/javac -version

    from the command prompt.

*************************************************************************
"""

        # macOS has a helpful system utility to determine JAVA_HOME. Try it.
        elif tools.host_os == "Darwin":
            tools.logger.debug(
                "Evaluating /usr/libexec/java_home...", prefix=cls.full_name
            )
            try:
                # If /usr/libexec/java_home doesn't exist, OSError will be raised
                # If no JRE/JDK is installed, /usr/libexec/java_home raises an error
                java_home = tools.subprocess.check_output(
                    ["/usr/libexec/java_home"],
                ).strip("\n")
            except (OSError, subprocess.CalledProcessError):
                tools.logger.debug("An existing JDK/JRE was not returned")
            else:
                try:
                    version_str = cls.version_from_path(tools, java_home)
                    if version_str.split(".")[0] == cls.JDK_MAJOR_VER:
                        java = JDK(tools, java_home=Path(java_home))
                except (OSError, subprocess.CalledProcessError, IndexError):
                    pass  # do not alert user if macOS found an unqualified JDK

        if java is None:
            # Inform the user if the user-specified JDK wasn't valid
            if install_message:
                tools.logger.warning(install_message)

            # Use the Briefcase JDK install
            java_home = tools.base_path / cls.JDK_INSTALL_DIR_NAME

            # The macOS download has a weird layout (inherited from the official Oracle
            # release). The actual JAVA_HOME is deeper inside the directory structure.
            if tools.host_os == "Darwin":
                java_home = java_home / "Contents/Home"

            java = JDK(tools=tools, java_home=java_home)

            if not java.exists():
                if install:
                    tools.logger.info(
                        f"A Java {cls.JDK_MAJOR_VER} JDK was not found; downloading and installing...",
                        prefix=cls.name,
                    )
                    tools.logger.info(
                        f"To use an existing JDK {cls.JDK_MAJOR_VER} instance, "
                        f"specify its root directory path in the JAVA_HOME environment variable."
                    )
                    tools.logger.info()
                    java.install()
                else:
                    raise MissingToolError("Java")

        tools.logger.debug(f"Using JDK at {java.java_home}")
        tools.java = java
        return java

    def exists(self) -> bool:
        return (self.java_home / "bin").exists()

    @property
    def managed_install(self) -> bool:
        try:
            # Determine if java_home is relative to the briefcase data directory.
            # If java_home isn't inside this directory, this will raise a ValueError,
            # indicating it is a non-managed install.
            self.java_home.relative_to(self.tools.base_path)
            return True
        except ValueError:
            return False

    def install(self):
        """Download and install a JDK."""
        jdk_zip_path = self.tools.file.download(
            url=self.OpenJDK_download_url,
            download_path=self.tools.base_path,
            role=f"Java {self.JDK_MAJOR_VER} JDK",
        )

        with self.tools.input.wait_bar("Installing OpenJDK..."):
            try:
                self.tools.file.unpack_archive(
                    os.fsdecode(jdk_zip_path),
                    extract_dir=os.fsdecode(self.tools.base_path),
                )
            except (shutil.ReadError, EOFError) as e:
                raise BriefcaseCommandError(
                    f"""\
Unable to unpack OpenJDK ZIP file. The download may have been interrupted
or corrupted.

Delete {jdk_zip_path} and run briefcase again.
"""
                ) from e

            jdk_zip_path.unlink()  # Zip file no longer needed once unpacked.

            # The tarball will unpack into <briefcase data dir>/tools/jdk-17.0.X+7
            # (or whatever name matches the current release).
            # We turn this into <briefcase data dir>/tools/java so we have a consistent name.
            java_unpack_path = (
                self.tools.base_path / f"jdk-{self.JDK_RELEASE}+{self.JDK_BUILD}"
            )
            java_unpack_path.rename(self.tools.base_path / self.JDK_INSTALL_DIR_NAME)

    def uninstall(self):
        """Uninstall a JDK."""
        with self.tools.input.wait_bar("Removing old JDK install..."):
            if self.tools.host_os == "Darwin":
                self.tools.shutil.rmtree(self.java_home.parent.parent)
            else:
                self.tools.shutil.rmtree(self.java_home)



from unittest.mock import MagicMock

import pytest

from briefcase.console import Console
from briefcase.integrations.android_sdk import AndroidSDK
from briefcase.integrations.base import ToolCache


@pytest.fixture
def mock_tools(tmp_path, mock_tools) -> ToolCache:
    mock_tools.input = MagicMock(spec_set=Console)

    # For default test purposes, assume we're on macOS x86_64
    mock_tools.host_os = "Darwin"
    mock_tools.host_arch = "x86_64"

    return mock_tools


@pytest.fixture
def android_sdk(android_sdk) -> AndroidSDK:
    # Mock some existing emulators
    android_sdk.emulators = MagicMock(
        return_value=[
            "runningEmulator",
            "idleEmulator",
        ]
    )
    return android_sdk


@pytest.mark.parametrize(
    "host_os, host_arch, emulator_abi",
    [
        ("Darwin", "x86_64", "x86_64"),
        ("Darwin", "arm64", "arm64-v8a"),
        ("Windows", "AMD64", "x86_64"),
        ("Linux", "x86_64", "x86_64"),
        ("Linux", "aarch64", "arm64-v8a"),
    ],
)
def test_create_emulator(
    mock_tools,
    android_sdk,
    tmp_path,
    host_os,
    host_arch,
    emulator_abi,
):
    """A new emulator can be created."""
    # This test validates everything going well on first run.
    # This means the skin will be downloaded and unpacked.

    # Mock the hardware and operating system to specific values
    mock_tools.host_os = host_os
    mock_tools.host_arch = host_arch

    # Mock the user providing several invalid names before getting it right.
    mock_tools.input.side_effect = [
        "runningEmulator",
        "invalid name",
        "annoying!",
        "new-emulator",
    ]

    # Mock the initial output of an AVD config file.
    avd_config_path = tmp_path / "home/.android/avd/new-emulator.avd/config.ini"
    avd_config_path.parent.mkdir(parents=True)
    with avd_config_path.open("w", encoding="utf-8") as f:
        f.write("hw.device.name=pixel\n")

    # Mock the internal emulator creation method
    android_sdk._create_emulator = MagicMock()

    # Create the emulator
    avd = android_sdk.create_emulator()

    # The expected device AVD was created.
    assert avd == "new-emulator"

    # The call was made to create the emulator
    android_sdk._create_emulator.assert_called_once_with(
        avd="new-emulator",
        device_type="pixel",
        skin="pixel_7_pro",
        system_image=f"system-images;android-31;default;{emulator_abi}",
    )


def test_default_name(mock_tools, android_sdk, tmp_path):
    """A new emulator can be created with the default name."""
    # This test doesn't validate most of the test process;
    # it only checks that the emulator is created with the default name.

    # User provides no input; default name will be used
    mock_tools.input.return_value = ""

    # Mock the internal emulator creation method
    android_sdk._create_emulator = MagicMock()

    # Create the emulator
    avd = android_sdk.create_emulator()

    # The expected device AVD was created.
    assert avd == "beePhone"


def test_default_name_with_collisions(mock_tools, android_sdk, tmp_path):
    """The default name will avoid collisions with existing emulators."""
    # This test doesn't validate most of the test process;
    # it only checks that the emulator is created with the default name.

    # Create some existing emulators that will collide with the default name.
    android_sdk.emulators = MagicMock(
        return_value=[
            "beePhone2",
            "runningEmulator",
            "beePhone",
        ]
    )
    # User provides no input; default name will be used
    mock_tools.input.return_value = ""

    # Mock the internal emulator creation method
    android_sdk._create_emulator = MagicMock()

    # Create the emulator
    avd = android_sdk.create_emulator()

    # The expected device AVD was created.
    assert avd == "beePhone3"



from pathlib import Path

import pytest

from briefcase.commands.create import cookiecutter_cache_path


@pytest.mark.parametrize(
    "template, cache_dir",
    [
        # Git/Github URLs
        ("https://github.com/beeware/template.git", "template"),
        ("https://github.com/beeware/template/", "template"),
        ("https://github.com/beeware/template", "template"),
        ("gh:beeware/template", "template"),
        ("git+ssh://git@github.com/beeware/template.git", "template"),
        ("git+https://beeware.org/template", "template"),
        # Paths are valid templates. They're not cached, but this
        # method returns as if they would be, and the "is it a repo"
        # check performed by git will fail instead.
        ("/path/to/template/", "template"),
        ("/path/to/template", "template"),
        ("path/to/template/", "template"),
        ("path/to/template", "template"),
        ("template/", "template"),
        ("template", "template"),
        # Zipfiles are also valid templates. Same rules apply as with paths.
        ("https://example.com/path/to/template.zip", "template.zip"),
        ("/path/to/template.zip", "template.zip"),
        ("path/to/template.zip", "template.zip"),
    ],
)
def test_cookiecutter_cache_path(template, cache_dir):
    """The cookiecutter cache path can be determined for various template types."""
    assert cookiecutter_cache_path(template) == (
        Path.home() / ".cookiecutters" / cache_dir
    )



from __future__ import annotations

import os
import shutil
from pathlib import Path

from briefcase.exceptions import BriefcaseCommandError, MissingToolError
from briefcase.integrations.base import ManagedTool, ToolCache


class WiX(ManagedTool):
    name = "wix"
    full_name = "WiX"
    supported_host_os = {"Windows"}

    def __init__(
        self,
        tools: ToolCache,
        wix_home: Path | None = None,
        bin_install: bool = False,
    ):
        """Create a wrapper around a WiX install.

        :param tools: ToolCache of available tools.
        :param wix_home: The path of the WiX installation.
        :param bin_install: Is the install a binaries-only install? A full MSI install
            of WiX has a `/bin` folder in the paths; a binaries-only install does not.
        :returns: A valid WiX SDK wrapper. If WiX is not available, and was not
            installed, raises MissingToolError.
        """
        super().__init__(tools=tools)
        if wix_home:
            self.wix_home = wix_home
        else:
            self.wix_home = tools.base_path / "wix"
        self.bin_install = bin_install

    @property
    def download_url(self) -> str:
        return "https://github.com/wixtoolset/wix3/releases/download/wix3141rtm/wix314-binaries.zip"

    @property
    def heat_exe(self) -> Path:
        if self.bin_install:
            return self.wix_home / "heat.exe"
        else:
            return self.wix_home / "bin/heat.exe"

    @property
    def light_exe(self) -> Path:
        if self.bin_install:
            return self.wix_home / "light.exe"
        else:
            return self.wix_home / "bin/light.exe"

    @property
    def candle_exe(self) -> Path:
        if self.bin_install:
            return self.wix_home / "candle.exe"
        else:
            return self.wix_home / "bin/candle.exe"

    @classmethod
    def verify_install(cls, tools: ToolCache, install: bool = True, **kwargs) -> WiX:
        """Verify that there is a WiX install available.

        If the WIX environment variable is set, that location will be checked for a
        valid WiX installation.

        If the location provided doesn't contain an SDK, or no location is provided, an
        SDK is downloaded.

        :param tools: ToolCache of available tools
        :param install: Should WiX be installed if it is not found?
        """
        # short circuit since already verified and available
        if hasattr(tools, "wix"):
            return tools.wix

        # Look for the WIX environment variable
        if wix_env := tools.os.environ.get("WIX"):
            tools.logger.debug("Evaluating WIX...", prefix=cls.full_name)
            tools.logger.debug(f"WIX={wix_env}")
            wix_home = Path(wix_env)

            # Set up the paths for the WiX executables we will use.
            wix = WiX(tools=tools, wix_home=wix_home)

            if not wix.exists():
                raise BriefcaseCommandError(
                    f"""\
The WIX environment variable:

{wix_home}

does not point to an install of the WiX Toolset.
"""
                )

        else:
            wix = WiX(tools=tools, bin_install=True)

            if not wix.exists():
                if install:
                    tools.logger.info(
                        "The WiX toolset was not found; downloading and installing...",
                        prefix=cls.name,
                    )
                    wix.install()
                else:
                    raise MissingToolError("WiX")

        tools.logger.debug(f"Using WiX at {wix.wix_home}")
        tools.wix = wix
        return wix

    def exists(self) -> bool:
        return (
            self.heat_exe.is_file()
            and self.light_exe.is_file()
            and self.candle_exe.is_file()
        )

    @property
    def managed_install(self) -> bool:
        try:
            # Determine if wix_home is relative to the briefcase data directory.
            # If wix_home isn't inside this directory, this will raise a ValueError,
            # indicating it is a non-managed install.
            self.wix_home.relative_to(self.tools.base_path)
            return True
        except ValueError:
            return False

    def install(self):
        """Download and install WiX."""
        wix_zip_path = self.tools.file.download(
            url=self.download_url,
            download_path=self.tools.base_path,
            role="WiX",
        )

        try:
            with self.tools.input.wait_bar("Installing WiX..."):
                self.tools.file.unpack_archive(
                    os.fsdecode(wix_zip_path),
                    extract_dir=os.fsdecode(self.wix_home),
                )
        except (shutil.ReadError, EOFError) as e:
            raise BriefcaseCommandError(
                f"""\
Unable to unpack WiX ZIP file. The download may have been
interrupted or corrupted.

Delete {wix_zip_path} and run briefcase again.
"""
            ) from e

        # Zip file no longer needed once unpacked.
        wix_zip_path.unlink()

    def uninstall(self):
        """Uninstall WiX."""
        with self.tools.input.wait_bar("Removing old WiX install..."):
            self.tools.shutil.rmtree(self.wix_home)



import pytest

from briefcase.exceptions import ParseError
from briefcase.platforms.linux import parse_freedesktop_os_release


def test_parse():
    "Content can be parsed from a freedesktop file."
    content = """
KEY1=value
KEY2="quoted value"
KEY3='another quoted value'

# Commented line
KEY4=42
"""
    assert parse_freedesktop_os_release(content) == {
        "KEY1": "value",
        "KEY2": "quoted value",
        "KEY3": "another quoted value",
        "KEY4": "42",
    }


@pytest.mark.parametrize(
    "content, error",
    [
        ("KEY=value\nnot valid content", r"Line 2: 'not valid content'"),
        ("KEY=value\nBAD='unbalanced quote", r"string literal"),
    ],
)
def test_parse_error(content, error):
    with pytest.raises(
        ParseError,
        match=error,
    ):
        parse_freedesktop_os_release(content)



import subprocess
from unittest.mock import ANY

import pytest

from briefcase.console import LogLevel

# These tests ignore the elsewhere-tested complexities of Dockerizing
# the arguments and just focuses on the semantics of a run() call.


@pytest.mark.usefixtures("mock_docker")
@pytest.mark.usefixtures("mock_docker_app_context")
def test_simple_call(mock_tools, my_app, tmp_path, sub_stream_kw, capsys):
    """A simple call will be invoked."""

    mock_tools[my_app].app_context.run(["hello", "world"])

    mock_tools[my_app].app_context._dockerize_args.assert_called_once_with(
        ["hello", "world"]
    )
    # calls to run() default to using Popen() due to output streaming
    mock_tools.subprocess._subprocess.Popen.assert_called_once_with(
        ANY,
        env={"DOCKER_CLI_HINTS": "false", "PROCESS_ENV_VAR": "VALUE"},
        **sub_stream_kw,
    )
    assert capsys.readouterr().out == (
        "\n"
        "Entering Docker context...\n"
        "Docker| ------------------------------------------------------------------\n"
        "Docker| ------------------------------------------------------------------\n"
        "Leaving Docker context.\n"
        "\n"
    )


@pytest.mark.usefixtures("mock_docker")
@pytest.mark.usefixtures("mock_docker_app_context")
def test_interactive(mock_tools, my_app, tmp_path, sub_kw, capsys):
    """Docker can be invoked in interactive mode."""
    mock_tools[my_app].app_context.run(["hello", "world"], interactive=True)

    # Interactive Docker runs must disable output streaming
    mock_tools[my_app].app_context._dockerize_args.assert_called_once_with(
        ["hello", "world"],
        interactive=True,
        stream_output=False,
    )
    # Interactive means the call to run is direct, rather than going through Popen
    mock_tools.subprocess._subprocess.run.assert_called_once_with(
        ANY,
        env={"DOCKER_CLI_HINTS": "false", "PROCESS_ENV_VAR": "VALUE"},
        **sub_kw,
    )
    assert capsys.readouterr().out == (
        "\n"
        "Entering Docker context...\n"
        "Docker| ------------------------------------------------------------------\n"
        "Docker| ------------------------------------------------------------------\n"
        "Leaving Docker context.\n"
        "\n"
    )


@pytest.mark.usefixtures("mock_docker")
@pytest.mark.usefixtures("mock_docker_app_context")
def test_call_with_extra_kwargs(
    mock_tools,
    my_app,
    tmp_path,
    sub_stream_kw,
    capsys,
):
    """Extra keyword arguments are passed through as-is; env modifications are
    converted."""

    mock_tools[my_app].app_context.run(
        ["hello", "world"],
        encoding="ISO-42",
        extra_kw="extra",
    )

    mock_tools[my_app].app_context._dockerize_args.assert_called_once_with(
        ["hello", "world"],
        encoding="ISO-42",
        extra_kw="extra",
    )
    mock_tools.subprocess._subprocess.Popen.assert_called_once_with(
        ANY,
        extra_kw="extra",
        encoding="ISO-42",
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        bufsize=1,
        text=True,
        errors="backslashreplace",
        env={"DOCKER_CLI_HINTS": "false", "PROCESS_ENV_VAR": "VALUE"},
    )
    assert capsys.readouterr().out == (
        "\n"
        "Entering Docker context...\n"
        "Docker| ------------------------------------------------------------------\n"
        "Docker| ------------------------------------------------------------------\n"
        "Leaving Docker context.\n"
        "\n"
    )


@pytest.mark.usefixtures("mock_docker")
@pytest.mark.usefixtures("mock_docker_app_context")
def test_simple_verbose_call(mock_tools, my_app, tmp_path, sub_stream_kw, capsys):
    """If verbosity is turned out, there is output."""
    mock_tools[my_app].app_context.tools.logger.verbosity = LogLevel.DEBUG

    mock_tools[my_app].app_context.run(["hello", "world"])

    mock_tools[my_app].app_context._dockerize_args.assert_called_once_with(
        ["hello", "world"],
    )
    mock_tools.subprocess._subprocess.Popen.assert_called_once_with(
        ANY,
        env={"DOCKER_CLI_HINTS": "false", "PROCESS_ENV_VAR": "VALUE"},
        **sub_stream_kw,
    )
    console_output = capsys.readouterr().out
    assert "Entering Docker context...\n" in console_output
    assert "Docker| >>> Running Command:\n" in console_output



import pytest

from briefcase.config import is_valid_app_name


@pytest.mark.parametrize(
    "name",
    [
        "helloworld",
        "helloWorld",
        "hello42world",
        "42helloworld",
        "hello_world",
        "hello-world",
    ],
)
def test_is_valid_app_name(name):
    """Test that valid app names are accepted."""
    assert is_valid_app_name(name)


@pytest.mark.parametrize(
    "name",
    [
        "hello world",
        "helloworld!",
        "_helloworld",
        "-helloworld",
        "switch",
        "pass",
        "false",
        "False",
        "YIELD",
        "main",
        "socket",
        "test",
        # ı, İ and K (i.e. 0x212a) are valid ASCII when made lowercase and as such are
        # accepted by the official PEP 508 regex... but they are rejected here to ensure
        # compliance with the regex that is used in practice.
        "helloworld_ı",
        "İstanbul",
        "Kelvin",
    ],
)
def test_is_invalid_app_name(name):
    """Test that invalid app names are rejected."""
    assert not is_valid_app_name(name)



from __future__ import annotations

from abc import ABC
from typing import Any, TypedDict


class AppContext(TypedDict):
    formal_name: str
    app_name: str
    class_name: str
    module_name: str
    project_name: str
    description: str
    author: str
    author_email: str
    bundle: str
    url: str
    license: str
    briefcase_version: str
    template_source: str
    template_branch: str


class BaseGuiBootstrap(ABC):
    """Definition for a plugin that defines a new Briefcase app."""

    # These are the field names that will be defined in the cookiecutter context.
    # Any fields defined here must be implemented as methods that return a ``str``
    # or ``None``. Returning ``None`` omits the field as a key in the context, thereby
    # deferring the value for the field to the cookiecutter template.
    fields: list[str] = [
        "app_source",
        "app_start_source",
        "pyproject_table_briefcase_extra_content",
        "pyproject_table_briefcase_app_extra_content",
        "pyproject_table_macOS",
        "pyproject_table_linux",
        "pyproject_table_linux_system_debian",
        "pyproject_table_linux_system_rhel",
        "pyproject_table_linux_system_suse",
        "pyproject_table_linux_system_arch",
        "pyproject_table_linux_appimage",
        "pyproject_table_linux_flatpak",
        "pyproject_table_windows",
        "pyproject_table_iOS",
        "pyproject_table_android",
        "pyproject_table_web",
        "pyproject_extra_content",
    ]

    # A short annotation that's appended to the name of the GUI toolkit when the user
    # is presented with the options to create a new project.
    display_name_annotation: str = ""

    def __init__(self, context: AppContext):
        # context contains metadata about the app being created
        self.context = context

    def extra_context(self) -> dict[str, Any] | None:
        """Runs prior to other plugin hooks to provide additional context.

        This can be used to prompt the user with additional questions or run arbitrary
        logic to supplement the context provided to cookiecutter.
        """

    def app_source(self) -> str | None:
        """The Python source code for app.py."""

    def app_start_source(self) -> str | None:
        """The Python source code for __main__.py to start the app."""

    def pyproject_table_briefcase_extra_content(self) -> str | None:
        """Additional content for ``tool.briefcase`` table."""

    def pyproject_table_briefcase_app_extra_content(self) -> str | None:
        """Additional content for ``tool.briefcase.app.<app-name>`` table."""

    def pyproject_table_macOS(self) -> str | None:
        """Content for ``tool.briefcase.app.<app-name>.macOS`` table."""

    def pyproject_table_linux(self) -> str | None:
        """Content for ``tool.briefcase.app.<app-name>.linux`` table."""

    def pyproject_table_linux_system_debian(self) -> str | None:
        """Content for ``tool.briefcase.app.<app-name>.linux.debian`` table."""

    def pyproject_table_linux_system_rhel(self) -> str | None:
        """Content for ``tool.briefcase.app.<app-name>.linux.rhel`` table."""

    def pyproject_table_linux_system_suse(self) -> str | None:
        """Content for ``tool.briefcase.app.<app-name>.linux.suse`` table."""

    def pyproject_table_linux_system_arch(self) -> str | None:
        """Content for ``tool.briefcase.app.<app-name>.linux.arch`` table."""

    def pyproject_table_linux_appimage(self) -> str | None:
        """Content for ``tool.briefcase.app.<app-name>.linux.appimage`` table."""

    def pyproject_table_linux_flatpak(self) -> str | None:
        """Content for ``tool.briefcase.app.<app-name>.linux.flatpak`` table."""

    def pyproject_table_windows(self) -> str | None:
        """Content for ``tool.briefcase.app.<app-name>.windows`` table."""

    def pyproject_table_iOS(self) -> str | None:
        """Content for ``tool.briefcase.app.<app-name>.iOS`` table."""

    def pyproject_table_android(self) -> str | None:
        """Content for ``tool.briefcase.app.<app-name>.android`` table."""

    def pyproject_table_web(self) -> str | None:
        """Content for ``tool.briefcase.app.<app-name>.web`` table."""

    def pyproject_extra_content(self) -> str | None:
        """Additional TOML to add to the bottom of pyproject.toml."""
